@@ -214,6 +214,7 @@ main(
 
     safe_fd(-1, 0);
     safe_cd();
+    make_crc_table();
 
     /*
      * Nexenta needs the SUN_PERSONALITY env variable to be unset, otherwise
@@ -456,7 +457,7 @@ main(
     }
 
 #ifndef SINGLE_USERID
-    if (geteuid() == 0) {
+    if (getuid() == 0) {
 	if (strcasecmp(auth, "krb5") != 0) {
 	    struct passwd *pwd;
 	    /* lookup our local user name */
