@@ -96,7 +96,7 @@ typedef struct handling_params_s {
     GSList *file_states;
 
     /* read buffer */
-    gpointer buf;
+    gchar *buf;
     gsize buf_size; /* allocated size */
     gsize buf_len; /* number of active bytes .. */
     gsize buf_offset; /* ..starting at buf + buf_offset */
@@ -116,7 +116,7 @@ struct amar_s {
 
     /* internal buffer; on writing, this is WRITE_BUFFER_SIZE bytes, and
      * always has at least RECORD_SIZE bytes free. */
-    gpointer buf;
+    gchar *buf;
     size_t buf_len;
     size_t buf_size;
     handling_params_t *hp;
@@ -554,7 +554,7 @@ amar_attr_add_data_buffer(
 			  rec_eoa, data, rec_data_size, error))
 	    return FALSE;
 
-	data += rec_data_size;
+	data = (gchar *)data + rec_data_size;
 	size -= rec_data_size;
 	attribute->size += rec_data_size;
     }
@@ -588,6 +588,7 @@ amar_attr_add_data_fd_thread(
     amar_attr_t *attribute = (amar_attr_t *)data;
 
     amar_attr_add_data_fd(attribute, attribute->fd, attribute->eoa, attribute->error);
+    close(attribute->fd);
     attribute->fd = -1;
     attribute->eoa = 0;
     attribute->error = NULL;
@@ -651,7 +652,7 @@ typedef struct attr_state_s {
     guint16  attrid;
     amar_attr_handling_t *handling;
     int      fd;
-    gpointer buf;
+    gchar *buf;
     gsize buf_len;
     gsize buf_size;
     gpointer attr_data;
@@ -784,7 +785,7 @@ retry:
       : buf_skip_((archive), (hp), (skipbytes)))
 
 /* Get a pointer to the current position in the buffer */
-#define buf_ptr(hp) ((hp)->buf + (hp)->buf_offset)
+#define buf_ptr(hp) ((gchar *)(hp)->buf + (hp)->buf_offset)
 
 /* Get the amount of data currently available in the buffer */
 #define buf_avail(hp) ((hp)->buf_len)
@@ -1238,8 +1239,11 @@ amar_read_cb(
 	hp->buf_offset = 0;
     }
 
-    if (count == -1 ||
-	(count == 0 && hp->buf_len == 0)) {
+    if (count == -1 || count == 0) {
+	if (count == 0 && hp->buf_len != 0) {
+	    g_set_error(hp->error, amar_error_quark(), EINVAL,
+			    "Archive ended with a partial record");
+	}
 	hp->got_eof = TRUE;
 	amar_stop_read(archive);
 
