@@ -58,6 +58,7 @@
 #include "conffile.h"
 #include "getopt.h"
 #include "event.h"
+#include "security-file.h"
 
 int debug_application = 1;
 #define application_debug(i, ...) do {	\
@@ -129,7 +130,8 @@ static void ambsdtar_set_timestamps(application_argument_t *argument,
 				    int                     level,
 				    char                   *timestamps,
 				    FILE                   *mesgstream);
-static GPtrArray *ambsdtar_build_argv(application_argument_t *argument,
+static GPtrArray *ambsdtar_build_argv(char *bsdtar_realpath,
+				      application_argument_t *argument,
 				      char *timestamps,
 				      char **file_exclude, char **file_include,
 				      int command);
@@ -142,6 +144,8 @@ static GSList *ignore_message = NULL;
 static GSList *strange_message = NULL;
 static char   *exit_handling;
 static int    exit_value[256];
+static int    exit_status = 0;
+static off_t  gblocksize = 0;
 
 static struct option long_options[] = {
     {"config"          , 1, NULL,  1},
@@ -419,6 +423,7 @@ main(
 		 break;
 	case 28: amfree(argument.tar_blocksize);
 		 argument.tar_blocksize = g_strdup(optarg);
+		 gblocksize = atoi(argument.tar_blocksize);
 		 break;
 	case 33: argument.command_options =
 			g_slist_append(argument.command_options,
@@ -463,7 +468,6 @@ main(
     for(i=0;i<256;i++)
 	exit_value[i] = 1; /* BAD  */
     exit_value[0] = 0;     /* GOOD */
-    exit_value[1] = 0;     /* GOOD */
     if (exit_handling) {
 	char *s = exit_handling;
 	while (s) {
@@ -537,7 +541,23 @@ main(
 
     dbclose();
 
-    return 0;
+    return exit_status;
+}
+
+static char *validate_command_options(
+    application_argument_t *argument)
+{
+    GSList    *copt;
+
+    for (copt = argument->command_options; copt != NULL; copt = copt->next) {
+	char *opt = (char *)copt->data;
+
+	if (g_str_has_prefix(opt, "--use-compress-program")) {
+	    return opt;
+	}
+    }
+
+    return NULL;
 }
 
 static void
@@ -570,6 +590,8 @@ static void
 ambsdtar_selfcheck(
     application_argument_t *argument)
 {
+    char *option;
+
     if (argument->dle.disk) {
 	char *qdisk = quote_string(argument->dle.disk);
 	fprintf(stdout, "OK disk %s\n", qdisk);
@@ -580,28 +602,37 @@ ambsdtar_selfcheck(
     ambsdtar_build_exinclude(&argument->dle, 1, NULL, NULL, NULL, NULL);
 
     printf("OK ambsdtar\n");
+
+    if ((option = validate_command_options(argument))) {
+	fprintf(stdout, "ERROR Invalid '%s' COMMAND-OPTIONS\n", option);
+    }
+
     if (bsdtar_path) {
 	if (check_file(bsdtar_path, X_OK)) {
-	    char *bsdtar_version;
-	    GPtrArray *argv_ptr = g_ptr_array_new();
+	    char *bsdtar_realpath;
+	    if (check_exec_for_suid("BSDTAR_PATH", bsdtar_path, stdout, &bsdtar_realpath)) {
+		char *bsdtar_version;
+		GPtrArray *argv_ptr = g_ptr_array_new();
 
-	    g_ptr_array_add(argv_ptr, bsdtar_path);
-	    g_ptr_array_add(argv_ptr, "--version");
-	    g_ptr_array_add(argv_ptr, NULL);
+		g_ptr_array_add(argv_ptr, bsdtar_realpath);
+		g_ptr_array_add(argv_ptr, "--version");
+		g_ptr_array_add(argv_ptr, NULL);
 
-	    bsdtar_version = get_first_line(argv_ptr);
-	    if (bsdtar_version) {
-		char *tv, *bv;
-		for (tv = bsdtar_version; *tv && !g_ascii_isdigit(*tv); tv++);
-		for (bv = tv; *bv && *bv != ' '; bv++);
-		if (*bv) *bv = '\0';
-		printf("OK ambsdtar bsdtar-version %s\n", tv);
-	    } else {
-		printf(_("ERROR [Can't get %s version]\n"), bsdtar_path);
+		bsdtar_version = get_first_line(argv_ptr);
+		if (bsdtar_version) {
+		    char *tv, *bv;
+		    for (tv = bsdtar_version; *tv && !g_ascii_isdigit(*tv); tv++);
+		    for (bv = tv; *bv && *bv != ' '; bv++);
+		    if (*bv) *bv = '\0';
+		    printf("OK ambsdtar bsdtar-version %s\n", tv);
+		} else {
+		    printf(_("ERROR [Can't get %s version]\n"), bsdtar_path);
+		}
+
+		g_ptr_array_free(argv_ptr, TRUE);
+		amfree(bsdtar_version);
 	    }
-
-	    g_ptr_array_free(argv_ptr, TRUE);
-	    amfree(bsdtar_version);
+	    amfree(bsdtar_realpath);
 	}
     } else {
 	printf(_("ERROR [BSDTAR program not available]\n"));
@@ -637,7 +668,6 @@ ambsdtar_estimate(
 {
     char      *incrname = NULL;
     GPtrArray *argv_ptr;
-    char      *cmd = NULL;
     int        nullfd = -1;
     int        pipefd = -1;
     FILE      *dumpout = NULL;
@@ -655,6 +685,8 @@ ambsdtar_estimate(
     char      *file_exclude = NULL;
     char      *file_include = NULL;
     GString   *strbuf;
+    char      *option;
+    char      *bsdtar_realpath;
 
     if (!argument->level) {
         fprintf(stderr, "ERROR No level argument\n");
@@ -669,10 +701,16 @@ ambsdtar_estimate(
         error(_("No device argument"));
     }
 
+    if ((option = validate_command_options(argument))) {
+	fprintf(stdout, "ERROR Invalid '%s' COMMAND-OPTIONS\n", option);
+	error("Invalid '%s' COMMAND-OPTIONS", option);
+    }
+
     if (argument->calcsize) {
 	char *dirname;
 	int   nb_exclude;
 	int   nb_include;
+
 	if (bsdtar_directory) {
 	    dirname = bsdtar_directory;
 	} else {
@@ -702,6 +740,11 @@ ambsdtar_estimate(
 	goto common_error;
     }
 
+    if (!check_exec_for_suid("BSDTAR_PATH", bsdtar_path, NULL, &bsdtar_realpath)) {
+	errmsg = g_strdup_printf("'%s' binary is not secure", bsdtar_path);
+	goto common_error;
+    }
+
     if (!state_dir) {
 	errmsg = g_strdup(_("STATE-DIR not defined"));
 	goto common_error;
@@ -712,8 +755,8 @@ ambsdtar_estimate(
 	char *timestamps;
 	level = GPOINTER_TO_INT(levels->data);
 	timestamps = ambsdtar_get_timestamps(argument, level, stdout, CMD_ESTIMATE);
-	cmd = g_strdup(bsdtar_path);
-	argv_ptr = ambsdtar_build_argv(argument, timestamps, &file_exclude,
+	argv_ptr = ambsdtar_build_argv(bsdtar_realpath,
+				       argument, timestamps, &file_exclude,
 				       &file_include, CMD_ESTIMATE);
 	amfree(timestamps);
 
@@ -725,7 +768,7 @@ ambsdtar_estimate(
 	    goto common_exit;
 	}
 
-	tarpid = pipespawnv(cmd, STDERR_PIPE, 1,
+	tarpid = pipespawnv(bsdtar_path, STDERR_PIPE, 1,
 			    &nullfd, &nullfd, &pipefd,
 			    (char **)argv_ptr->pdata);
 
@@ -749,9 +792,16 @@ ambsdtar_estimate(
 	    for(rp = re_table; rp->regex != NULL; rp++) {
 		if(match(rp->regex, line)) {
 		    if (rp->typ == DMP_SIZE) {
+			off_t blocksize = gblocksize;
 			size = ((the_num(line, rp->field)*rp->scale+1023.0)/1024.0);
 			if(size < 0.0)
 			    size = 1.0;             /* found on NeXT -- sigh */
+			if (!blocksize) {
+			    blocksize = 20;
+			}
+			blocksize /= 2;
+			size = (size+blocksize-1) / blocksize;
+			size *= blocksize;
 		    }
 		    break;
 		}
@@ -769,12 +819,12 @@ ambsdtar_estimate(
 		 level,
 		 walltime_str(timessub(curclock(), start_time)));
 	if(size == (off_t)-1) {
-	    errmsg = g_strdup_printf(_("no size line match in %s output"), cmd);
+	    errmsg = g_strdup_printf(_("no size line match in %s output"), bsdtar_path);
 	    g_debug(_("%s for %s"), errmsg, qdisk);
 	    g_debug(".....");
 	} else if(size == (off_t)0 && argument->level == 0) {
 	    g_debug(_("possible %s problem -- is \"%s\" really empty?"),
-		     cmd, argument->dle.disk);
+		     bsdtar_path, argument->dle.disk);
 	    g_debug(".....");
 	}
 	g_debug(_("estimate size for %s level %d: %lld KB"),
@@ -784,29 +834,32 @@ ambsdtar_estimate(
 
 	kill(-tarpid, SIGTERM);
 
-	g_debug(_("waiting for %s \"%s\" child"), cmd, qdisk);
+	g_debug(_("waiting for %s \"%s\" child"), bsdtar_path, qdisk);
 	waitpid(tarpid, &wait_status, 0);
 	if (WIFSIGNALED(wait_status)) {
 	    strbuf = g_string_new(errmsg);
 	    g_string_append_printf(strbuf, "%s terminated with signal %d: see %s",
-                cmd, WTERMSIG(wait_status), dbfn());
+                bsdtar_path, WTERMSIG(wait_status), dbfn());
 	    g_free(errmsg);
             errmsg = g_string_free(strbuf, FALSE);
+	    exit_status = 1;
 	} else if (WIFEXITED(wait_status)) {
 	    if (exit_value[WEXITSTATUS(wait_status)] == 1) {
                 strbuf = g_string_new(errmsg);
                 g_string_append_printf(strbuf, "%s exited with status %d: see %s",
-                    cmd, WEXITSTATUS(wait_status), dbfn());
+                    bsdtar_path, WEXITSTATUS(wait_status), dbfn());
 		g_free(errmsg);
                 errmsg = g_string_free(strbuf, FALSE);
+		exit_status = 1;
 	    } else {
 		/* Normal exit */
 	    }
 	} else {
 	    errmsg = g_strdup_printf(_("%s got bad exit: see %s"),
-				cmd, dbfn());
+				bsdtar_path, dbfn());
+	    exit_status = 1;
 	}
-	g_debug(_("after %s %s wait"), cmd, qdisk);
+	g_debug(_("after %s %s wait"), bsdtar_path, qdisk);
 
 common_exit:
 	if (errmsg) {
@@ -823,7 +876,7 @@ common_exit:
         }
 
 	g_ptr_array_free_full(argv_ptr);
-	amfree(cmd);
+	amfree(bsdtar_path);
 	amfree(incrname);
 
 	aclose(nullfd);
@@ -844,6 +897,7 @@ common_error:
     amfree(qdisk);
     g_debug("%s", errmsg);
     fprintf(stdout, "ERROR %s\n", qerrmsg);
+    exit_status = 1;
     amfree(file_exclude);
     amfree(file_include);
     amfree(errmsg);
@@ -860,6 +914,7 @@ typedef struct filter_s {
     gint64 size;		/* number of byte use in the buffer */
     gint64 allocated_size;	/* allocated size of the buffer     */
     event_handle_t *event;
+    int    out;
 } filter_t;
 
 static off_t dump_size = -1;
@@ -977,7 +1032,16 @@ read_text(
 		}
 	    }
 	    if(rp->typ == DMP_SIZE) {
+		off_t blocksize = gblocksize;
 		dump_size = (off_t)((the_num(line, rp->field)* rp->scale+1023.0)/1024.0);
+		if(dump_size < 0.0)
+		    dump_size = 1.0;             /* found on NeXT -- sigh */
+		if (!blocksize) {
+		    blocksize = 20;
+		}
+		blocksize /= 2;
+		dump_size = (dump_size+blocksize-1) / blocksize;
+		dump_size *= blocksize;
 	    }
 	    switch(rp->typ) {
 	    case DMP_NORMAL:
@@ -1031,7 +1095,6 @@ ambsdtar_backup(
     application_argument_t *argument)
 {
     int         dumpin;
-    char      *cmd = NULL;
     char      *qdisk;
     char      *timestamps;
     int        mesgf = 3;
@@ -1049,6 +1112,8 @@ ambsdtar_backup(
     char      *file_exclude;
     char      *file_include;
     char       new_timestamps[64];
+    char      *option;
+    char      *bsdtar_realpath;
 
     mesgstream = fdopen(mesgf, "w");
     if (!mesgstream) {
@@ -1056,8 +1121,20 @@ ambsdtar_backup(
     }
 
     if (!bsdtar_path) {
+	fprintf(mesgstream, "? BSDTAR-PATH not defined");
 	error(_("BSDTAR-PATH not defined"));
     }
+
+    if (!check_exec_for_suid("BSDTAR_PATH", bsdtar_path, NULL, &bsdtar_realpath)) {
+	fprintf(mesgstream, "? '%s' binary is not secure\n", bsdtar_path);
+	error("'%s' binary is not secure", bsdtar_path);
+    }
+
+    if ((option = validate_command_options(argument))) {
+	fprintf(mesgstream, "? Invalid '%s' COMMAND-OPTIONS\n", option);
+	error("Invalid '%s' COMMAND-OPTIONS", option);
+    }
+
     if (!state_dir) {
 	error(_("STATE-DIR not defined"));
     }
@@ -1083,12 +1160,12 @@ ambsdtar_backup(
     timestamps = ambsdtar_get_timestamps(argument,
 				   GPOINTER_TO_INT(argument->level->data),
 				   mesgstream, CMD_BACKUP);
-    cmd = g_strdup(bsdtar_path);
-    argv_ptr = ambsdtar_build_argv(argument, timestamps, &file_exclude,
+    argv_ptr = ambsdtar_build_argv(bsdtar_realpath,
+				   argument, timestamps, &file_exclude,
 				   &file_include, CMD_BACKUP);
 
     if (argument->dle.create_index) {
-	tarpid = pipespawnv(cmd, STDIN_PIPE|STDOUT_PIPE|STDERR_PIPE, 1,
+	tarpid = pipespawnv(bsdtar_path, STDIN_PIPE|STDOUT_PIPE|STDERR_PIPE, 1,
 			&dumpin, &data_out, &outf, (char **)argv_ptr->pdata);
 	g_ptr_array_free_full(argv_ptr);
 	argv_ptr = g_ptr_array_new();
@@ -1096,7 +1173,7 @@ ambsdtar_backup(
 	g_ptr_array_add(argv_ptr, g_strdup("tf"));
 	g_ptr_array_add(argv_ptr, g_strdup("-"));
 	g_ptr_array_add(argv_ptr, NULL);
-	indexpid = pipespawnv(cmd, STDIN_PIPE|STDOUT_PIPE|STDERR_PIPE, 1,
+	indexpid = pipespawnv(bsdtar_path, STDIN_PIPE|STDOUT_PIPE|STDERR_PIPE, 1,
 			  &index_in, &index_out, &index_err,
 			  (char **)argv_ptr->pdata);
 	g_ptr_array_free_full(argv_ptr);
@@ -1112,7 +1189,7 @@ ambsdtar_backup(
 	read_fd(index_out, "index out", &read_text);
 	read_fd(index_err, "index_err", &read_text);
     } else {
-	tarpid = pipespawnv(cmd, STDIN_PIPE|STDERR_PIPE, 1,
+	tarpid = pipespawnv(bsdtar_path, STDIN_PIPE|STDERR_PIPE, 1,
 			&dumpin, &dataf, &outf, (char **)argv_ptr->pdata);
 	g_ptr_array_free_full(argv_ptr);
 	aclose(dumpin);
@@ -1125,40 +1202,47 @@ ambsdtar_backup(
     waitpid(tarpid, &wait_status, 0);
     if (WIFSIGNALED(wait_status)) {
 	errmsg = g_strdup_printf(_("%s terminated with signal %d: see %s"),
-			    cmd, WTERMSIG(wait_status), dbfn());
+			    bsdtar_path, WTERMSIG(wait_status), dbfn());
+	exit_status = 1;
     } else if (WIFEXITED(wait_status)) {
 	if (exit_value[WEXITSTATUS(wait_status)] == 1) {
 	    errmsg = g_strdup_printf(_("%s exited with status %d: see %s"),
-				cmd, WEXITSTATUS(wait_status), dbfn());
+				bsdtar_path, WEXITSTATUS(wait_status), dbfn());
+	    exit_status = 1;
 	} else {
 	    /* Normal exit */
 	}
     } else {
 	errmsg = g_strdup_printf(_("%s got bad exit: see %s"),
-			    cmd, dbfn());
+			    bsdtar_path, dbfn());
+	exit_status = 1;
     }
     if (argument->dle.create_index) {
 	waitpid(indexpid, &wait_status, 0);
 	if (WIFSIGNALED(wait_status)) {
 	    errmsg = g_strdup_printf(_("'%s index' terminated with signal %d: see %s"),
-			    cmd, WTERMSIG(wait_status), dbfn());
+			    bsdtar_path, WTERMSIG(wait_status), dbfn());
+	    exit_status = 1;
 	} else if (WIFEXITED(wait_status)) {
 	    if (exit_value[WEXITSTATUS(wait_status)] == 1) {
 		errmsg = g_strdup_printf(_("'%s index' exited with status %d: see %s"),
-				cmd, WEXITSTATUS(wait_status), dbfn());
+				bsdtar_path, WEXITSTATUS(wait_status), dbfn());
+		exit_status = 1;
 	    } else {
 		/* Normal exit */
 	    }
 	} else {
 	    errmsg = g_strdup_printf(_("'%s index' got bad exit: see %s"),
-			    cmd, dbfn());
+			    bsdtar_path, dbfn());
+	    exit_status = 1;
 	}
     }
-    g_debug(_("after %s %s wait"), cmd, qdisk);
-    g_debug(_("ambsdtar: %s: pid %ld"), cmd, (long)tarpid);
+    g_debug(_("after %s %s wait"), bsdtar_path, qdisk);
+    g_debug(_("ambsdtar: %s: pid %ld"), bsdtar_path, (long)tarpid);
     if (errmsg) {
 	g_debug("%s", errmsg);
 	g_fprintf(mesgstream, "sendbackup: error [%s]\n", errmsg);
+	exit_status = 1;
     } else if (argument->dle.record) {
 	ambsdtar_set_timestamps(argument,
 				GPOINTER_TO_INT(argument->level->data),
@@ -1188,29 +1272,183 @@ ambsdtar_backup(
     amfree(file_include);
     amfree(timestamps);
     amfree(qdisk);
-    amfree(cmd);
+    amfree(bsdtar_path);
     amfree(errmsg);
 }
 
+static ssize_t
+read_filter_buffer(
+    filter_t *filter)
+{
+    ssize_t nread;
+
+    if (filter->buffer == NULL) {
+	/* allocate initial buffer */
+	filter->buffer = g_malloc(2048);
+	filter->first = 0;
+	filter->size = 0;
+	filter->allocated_size = 2048;
+    } else if (filter->first > 0) {
+	if (filter->allocated_size - filter->size - filter->first < 1024) {
+	    memmove(filter->buffer, filter->buffer + filter->first,
+				    filter->size);
+	    filter->first = 0;
+	}
+    } else if (filter->allocated_size - filter->size < 1024) {
+	/* double the size of the buffer */
+	filter->allocated_size *= 2;
+	filter->buffer = g_realloc(filter->buffer, filter->allocated_size);
+    }
+
+    nread = read(filter->fd, filter->buffer + filter->first + filter->size,
+		 filter->allocated_size - filter->first - filter->size - 2);
+
+    if (nread <= 0) {
+	event_release(filter->event);
+	aclose(filter->fd);
+	if (filter->size > 0 && filter->buffer[filter->first + filter->size - 1] != '\n') {
+	    /* Add a '\n' at end of buffer */
+	    filter->buffer[filter->first + filter->size] = '\n';
+	    filter->size++;
+	}
+    } else {
+	filter->size += nread;
+    }
+
+    return nread;
+}
+
+static void
+handle_restore_stdin(
+    void *cookie)
+{
+    filter_t *filter = cookie;
+    ssize_t   nread;
+
+    if (filter->buffer == NULL) {
+	/* allocate initial buffer */
+	filter->buffer = g_malloc(65536);
+	filter->first = 0;
+	filter->size = 0;
+	filter->allocated_size = 65536;
+    }
+    nread = read(filter->fd, filter->buffer, filter->allocated_size);
+
+    if (nread > 0) {
+	/* process the complete buffer */
+	int nwrite = full_write(filter->out , filter->buffer, nread);
+	if (nwrite < nread) {
+	    exit_status = 1;
+	    g_debug("wrote only %d bytes", nwrite);
+	    event_release(filter->event);
+	    filter->event = NULL;
+	    aclose(filter->fd);
+	    g_free(filter->buffer);
+	    filter->buffer = NULL;
+	    aclose(filter->out);
+	}
+    } else {
+	event_release(filter->event);
+	filter->event = NULL;
+	aclose(filter->fd);
+	g_free(filter->buffer);
+	filter->buffer = NULL;
+	aclose(filter->out);
+    }
+}
+
+static void
+handle_restore_stdout(
+    void *cookie)
+{
+    filter_t *filter = cookie;
+    ssize_t   nread;
+    char     *b, *p;
+    gint64    len;
+
+    nread = read_filter_buffer(filter);
+
+    /* process all complete lines */
+    b = filter->buffer + filter->first;
+    b[filter->size] = '\0';
+    while (b < filter->buffer + filter->first + filter->size &&
+	   (p = strchr(b, '\n')) != NULL) {
+	*p = '\0';
+	g_fprintf(stdout, "%s\n", b);
+	len = p - b + 1;
+	filter->first += len;
+	filter->size -= len;
+	b = p + 1;
+    }
+
+    if (nread <= 0) {
+	g_free(filter->buffer);
+    }
+}
+
+static void
+handle_restore_stderr(
+    void *cookie)
+{
+    filter_t *filter = cookie;
+    ssize_t   nread;
+    char     *b, *p;
+    gint64    len;
+
+    nread = read_filter_buffer(filter);
+
+    /* process all complete lines */
+    b = filter->buffer + filter->first;
+    b[filter->size] = '\0';
+    while (b < filter->buffer + filter->first + filter->size &&
+	   (p = strchr(b, '\n')) != NULL) {
+	*p = '\0';
+	if (*b == 'x' && *(b+1) == ' ') {
+	    g_fprintf(stdout, "%s\n", b+2);
+	} else {
+	    g_fprintf(stderr, "%s\n", b);
+	}
+	len = p - b + 1;
+	filter->first += len;
+	filter->size -= len;
+	b = p + 1;
+    }
+
+    if (nread <= 0) {
+	g_free(filter->buffer);
+    }
+}
+
 static void
 ambsdtar_restore(
     application_argument_t *argument)
 {
-    char       *cmd;
     GPtrArray  *argv_ptr = g_ptr_array_new();
-    char      **env;
     int         j;
-    char       *e;
     char       *include_filename = NULL;
     char       *exclude_filename = NULL;
     int         tarpid;
+    filter_t    in_buf;
+    filter_t    out_buf;
+    filter_t    err_buf;
+    int         tarin, tarout, tarerr;
+    char       *errmsg = NULL;
+    amwait_t    wait_status;
+    char       *bsdtar_realpath;
 
     if (!bsdtar_path) {
 	error(_("BSDTAR-PATH not defined"));
     }
 
-    cmd = g_strdup(bsdtar_path);
-    g_ptr_array_add(argv_ptr, g_strdup(bsdtar_path));
+    if (!check_exec_for_suid("BSDTAR_PATH", bsdtar_path, NULL, &bsdtar_realpath)) {
+	error("'%s' binary is not secure", bsdtar_path);
+    }
+
+    if (!security_allow_to_restore(NULL)) {
+	error("The user is not allowed to restore files");
+    }
+
+    g_ptr_array_add(argv_ptr, g_strdup(bsdtar_realpath));
     g_ptr_array_add(argv_ptr, g_strdup("--numeric-owner"));
     /* ignore trailing zero blocks on input (this was the default until tar-1.21) */
     if (argument->tar_blocksize) {
@@ -1369,29 +1607,67 @@ ambsdtar_restore(
 
     debug_executing(argv_ptr);
 
-    tarpid = fork();
-    switch (tarpid) {
-    case -1: error(_("%s: fork returned: %s"), get_pname(), strerror(errno));
-    case 0:
-	env = safe_env();
-	become_root();
-	execve(cmd, (char **)argv_ptr->pdata, env);
-	execv(cmd, (char **)argv_ptr->pdata);
-	e = strerror(errno);
-	error(_("error [exec %s: %s]"), cmd, e);
-	break;
-    default: break;
+    tarpid = pipespawnv(bsdtar_path, STDIN_PIPE|STDOUT_PIPE|STDERR_PIPE, 1,
+			&tarin, &tarout, &tarerr, (char **)argv_ptr->pdata);
+
+    in_buf.fd = 0;
+    in_buf.out = tarin;
+    in_buf.name = "stdin";
+    in_buf.buffer = NULL;
+    in_buf.first = 0;
+    in_buf.size = 0;
+    in_buf.allocated_size = 0;
+
+    out_buf.fd = tarout;
+    out_buf.name = "stdout";
+    out_buf.buffer = NULL;
+    out_buf.first = 0;
+    out_buf.size = 0;
+    out_buf.allocated_size = 0;
+
+    err_buf.fd = tarerr;
+    err_buf.name = "stderr";
+    err_buf.buffer = NULL;
+    err_buf.first = 0;
+    err_buf.size = 0;
+    err_buf.allocated_size = 0;
+
+    in_buf.event = event_register((event_id_t)0, EV_READFD,
+			    handle_restore_stdin, &in_buf);
+    out_buf.event = event_register((event_id_t)tarout, EV_READFD,
+			    handle_restore_stdout, &out_buf);
+    err_buf.event = event_register((event_id_t)tarerr, EV_READFD,
+			    handle_restore_stderr, &err_buf);
+
+    event_loop(0);
+
+    waitpid(tarpid, &wait_status, 0);
+    if (WIFSIGNALED(wait_status)) {
+	errmsg = g_strdup_printf(_("%s terminated with signal %d: see %s"),
+				 bsdtar_path, WTERMSIG(wait_status), dbfn());
+	exit_status = 1;
+    } else if (WIFEXITED(wait_status) && WEXITSTATUS(wait_status) > 0) {
+	errmsg = g_strdup_printf(_("%s exited with status %d: see %s"),
+				 bsdtar_path, WEXITSTATUS(wait_status), dbfn());
+	exit_status = 1;
+    }
+
+    g_debug(_("ambsdtar: %s: pid %ld"), bsdtar_path, (long)tarpid);
+    if (errmsg) {
+	g_debug("%s", errmsg);
+	fprintf(stderr, "error [%s]\n", errmsg);
     }
 
-    waitpid(tarpid, NULL, 0);
     if (argument->verbose == 0) {
 	if (exclude_filename)
 	    unlink(exclude_filename);
 	unlink(include_filename);
     }
-    amfree(cmd);
+    amfree(bsdtar_path);
     amfree(include_filename);
     amfree(exclude_filename);
+
+    g_free(errmsg);
 }
 
 static void
@@ -1694,6 +1970,7 @@ ambsdtar_set_timestamps(
 
 static GPtrArray *
 ambsdtar_build_argv(
+    char  *bsdtar_realpath,
     application_argument_t *argument,
     char  *timestamps,
     char **file_exclude,
@@ -1717,7 +1994,7 @@ ambsdtar_build_argv(
 	dirname = argument->dle.device;
     }
 
-    g_ptr_array_add(argv_ptr, g_strdup(bsdtar_path));
+    g_ptr_array_add(argv_ptr, g_strdup(bsdtar_realpath));
 
     g_ptr_array_add(argv_ptr, g_strdup("--create"));
     g_ptr_array_add(argv_ptr, g_strdup("--file"));
