@@ -71,6 +71,7 @@
 #include "client_util.h"
 #include "conffile.h"
 #include "getopt.h"
+#include "security-file.h"
 
 int debug_application = 1;
 #define application_debug(i, ...) do {	\
@@ -143,7 +144,8 @@ static void amgtar_build_exinclude(dle_t *dle, int verbose,
 static char *amgtar_get_incrname(application_argument_t *argument, int level,
 				 FILE *mesgstream, int command);
 static void check_no_check_device(void);
-static GPtrArray *amgtar_build_argv(application_argument_t *argument,
+static GPtrArray *amgtar_build_argv(char *gnutar_realpath,
+				application_argument_t *argument,
 				char *incrname, char **file_exclude,
 				char **file_include, int command);
 static char *gnutar_path;
@@ -157,6 +159,7 @@ static int gnutar_xattrs;
 static int gnutar_checkdevice;
 static int gnutar_no_unquote;
 static int gnutar_sparse;
+static int gnutar_sparse_set = 0;
 static GSList *normal_message = NULL;
 static GSList *ignore_message = NULL;
 static GSList *strange_message = NULL;
@@ -417,7 +420,8 @@ main(
 		 else if (strcasecmp(command, "selfcheck") == 0)
 		     printf(_("ERROR [%s: bad ONE-FILE-SYSTEM property value (%s)]\n"), get_pname(), optarg);
 		 break;
-	case 13: if (optarg && strcasecmp(optarg, "NO") == 0)
+	case 13: gnutar_sparse_set = 1;
+		 if (optarg && strcasecmp(optarg, "NO") == 0)
 		     gnutar_sparse = 0;
 		 else if (optarg && strcasecmp(optarg, "YES") == 0)
 		     gnutar_sparse = 1;
@@ -623,6 +627,27 @@ main(
     return 0;
 }
 
+static char *validate_command_options(
+    application_argument_t *argument)
+{
+    GSList    *copt;
+
+    for (copt = argument->command_options; copt != NULL; copt = copt->next) {
+	char *opt = (char *)copt->data;
+
+	if (g_str_has_prefix(opt, "--rsh-command") ||
+	    g_str_has_prefix(opt,"--to-command") ||
+	    g_str_has_prefix(opt,"--info-script") ||
+	    g_str_has_prefix(opt,"--new-volume-script") ||
+	    g_str_has_prefix(opt,"--rmt-command") ||
+	    g_str_has_prefix(opt,"--use-compress-program")) {
+	    return opt;
+	}
+    }
+
+    return NULL;
+}
+
 static void
 amgtar_support(
     application_argument_t *argument)
@@ -655,6 +680,8 @@ static void
 amgtar_selfcheck(
     application_argument_t *argument)
 {
+    char *option;
+
     if (argument->dle.disk) {
 	char *qdisk = quote_string(argument->dle.disk);
 	fprintf(stdout, "OK disk %s\n", qdisk);
@@ -665,12 +692,20 @@ amgtar_selfcheck(
     amgtar_build_exinclude(&argument->dle, 1, NULL, NULL, NULL, NULL);
 
     printf("OK amgtar\n");
+
+
+    if ((option = validate_command_options(argument))) {
+	fprintf(stdout, "ERROR [Invalid '%s' COMMAND-OPTIONS\n", option);
+    }
+
     if (gnutar_path) {
-	if (check_file(gnutar_path, X_OK)) {
+	char *gnutar_realpath = NULL;
+	if (check_file(gnutar_path, X_OK) &&
+	    check_exec_for_suid("GNUTAR_PATH", gnutar_path, stdout, &gnutar_realpath)) {
 	    char *gtar_version;
 	    GPtrArray *argv_ptr = g_ptr_array_new();
 
-	    g_ptr_array_add(argv_ptr, gnutar_path);
+	    g_ptr_array_add(argv_ptr, gnutar_realpath);
 	    g_ptr_array_add(argv_ptr, "--version");
 	    g_ptr_array_add(argv_ptr, NULL);
 
@@ -680,16 +715,16 @@ amgtar_selfcheck(
 		for (gv = gtar_version; *gv && !g_ascii_isdigit(*gv); gv++);
 		printf("OK amgtar gtar-version %s\n", gv);
 	    } else {
-		printf(_("ERROR [Can't get %s version]\n"), gnutar_path);
+		printf(_("ERROR [Can't get %s version]\n"), gnutar_realpath);
 	    }
 
 	    g_ptr_array_free(argv_ptr, TRUE);
 	    amfree(gtar_version);
 	}
+	amfree(gnutar_realpath);
     } else {
 	printf(_("ERROR [GNUTAR program not available]\n"));
     }
-
     if (gnutar_listdir && strlen(gnutar_listdir) == 0)
 	gnutar_listdir = NULL;
     if (gnutar_listdir) {
@@ -719,7 +754,6 @@ amgtar_estimate(
 {
     char      *incrname = NULL;
     GPtrArray *argv_ptr;
-    char      *cmd = NULL;
     int        nullfd = -1;
     int        pipefd = -1;
     FILE      *dumpout = NULL;
@@ -736,6 +770,8 @@ amgtar_estimate(
     GSList    *levels;
     char      *file_exclude;
     char      *file_include;
+    char      *option;
+    char      *gnutar_realpath = NULL;
 
     if (!argument->level) {
         fprintf(stderr, "ERROR No level argument\n");
@@ -752,6 +788,11 @@ amgtar_estimate(
 
     qdisk = quote_string(argument->dle.disk);
 
+    if ((option = validate_command_options(argument))) {
+	fprintf(stderr, "ERROR Invalid '%s' COMMAND-OPTIONS\n", option);
+	error("Invalid '%s' COMMAND-OPTIONS", option);
+    }
+
     if (argument->calcsize) {
 	char *dirname;
 	int   nb_exclude;
@@ -762,6 +803,7 @@ amgtar_estimate(
 	} else {
 	    dirname = argument->dle.device;
 	}
+
 	amgtar_build_exinclude(&argument->dle, 1,
 			       &nb_exclude, &file_exclude,
 			       &nb_include, &file_include);
@@ -783,6 +825,11 @@ amgtar_estimate(
 	goto common_error;
     }
 
+    if (!check_exec_for_suid("GNUTAR_PATH", gnutar_path, stderr, &gnutar_realpath)) {
+	errmsg = g_strdup_printf("'%s' binary is not secure", gnutar_path);
+	goto common_error;
+    }
+
     if (!gnutar_listdir) {
 	errmsg = vstrallocf(_("GNUTAR-LISTDIR not defined"));
 	goto common_error;
@@ -791,8 +838,8 @@ amgtar_estimate(
     for (levels = argument->level; levels != NULL; levels = levels->next) {
 	level = GPOINTER_TO_INT(levels->data);
 	incrname = amgtar_get_incrname(argument, level, stdout, CMD_ESTIMATE);
-	cmd = stralloc(gnutar_path);
-	argv_ptr = amgtar_build_argv(argument, incrname, &file_exclude,
+	argv_ptr = amgtar_build_argv(gnutar_realpath,
+				     argument, incrname, &file_exclude,
 				     &file_include, CMD_ESTIMATE);
 
 	start_time = curclock();
@@ -803,7 +850,7 @@ amgtar_estimate(
 	    goto common_exit;
 	}
 
-	tarpid = pipespawnv(cmd, STDERR_PIPE, 1,
+	tarpid = pipespawnv(gnutar_realpath, STDERR_PIPE, 1,
 			    &nullfd, &nullfd, &pipefd,
 			    (char **)argv_ptr->pdata);
 
@@ -845,12 +892,12 @@ amgtar_estimate(
 		 level,
 		 walltime_str(timessub(curclock(), start_time)));
 	if(size == (off_t)-1) {
-	    errmsg = vstrallocf(_("no size line match in %s output"), cmd);
+	    errmsg = vstrallocf(_("no size line match in %s output"), gnutar_realpath);
 	    dbprintf(_("%s for %s\n"), errmsg, qdisk);
 	    dbprintf(".....\n");
 	} else if(size == (off_t)0 && argument->level == 0) {
 	    dbprintf(_("possible %s problem -- is \"%s\" really empty?\n"),
-		     cmd, argument->dle.disk);
+		     gnutar_realpath, argument->dle.disk);
 	    dbprintf(".....\n");
 	}
 	dbprintf(_("estimate size for %s level %d: %lld KB\n"),
@@ -860,23 +907,23 @@ amgtar_estimate(
 
 	kill(-tarpid, SIGTERM);
 
-	dbprintf(_("waiting for %s \"%s\" child\n"), cmd, qdisk);
+	dbprintf(_("waiting for %s \"%s\" child\n"), gnutar_realpath, qdisk);
 	waitpid(tarpid, &wait_status, 0);
 	if (WIFSIGNALED(wait_status)) {
 	    errmsg = vstrallocf(_("%s terminated with signal %d: see %s"),
-				cmd, WTERMSIG(wait_status), dbfn());
+				gnutar_realpath, WTERMSIG(wait_status), dbfn());
 	} else if (WIFEXITED(wait_status)) {
 	    if (exit_value[WEXITSTATUS(wait_status)] == 1) {
 		errmsg = vstrallocf(_("%s exited with status %d: see %s"),
-				    cmd, WEXITSTATUS(wait_status), dbfn());
+				    gnutar_realpath, WEXITSTATUS(wait_status), dbfn());
 	    } else {
 		/* Normal exit */
 	    }
 	} else {
 	    errmsg = vstrallocf(_("%s got bad exit: see %s"),
-				cmd, dbfn());
+				gnutar_realpath, dbfn());
 	}
-	dbprintf(_("after %s %s wait\n"), cmd, qdisk);
+	dbprintf(_("after %s %s wait\n"), gnutar_realpath, qdisk);
 
 common_exit:
 	if (errmsg) {
@@ -896,7 +943,6 @@ common_exit:
         }
 
 	g_ptr_array_free_full(argv_ptr);
-	amfree(cmd);
 
 	aclose(nullfd);
 	afclose(dumpout);
@@ -904,6 +950,7 @@ common_exit:
 	fprintf(stdout, "%d %lld 1\n", level, (long long)size);
     }
     amfree(qdisk);
+    amfree(gnutar_realpath);
     return;
 
 common_error:
@@ -913,6 +960,7 @@ common_error:
     fprintf(stdout, "ERROR %s\n", qerrmsg);
     amfree(errmsg);
     amfree(qerrmsg);
+    amfree(gnutar_realpath);
     return;
 }
 
@@ -921,7 +969,6 @@ amgtar_backup(
     application_argument_t *argument)
 {
     int         dumpin;
-    char      *cmd = NULL;
     char      *qdisk;
     char      *incrname;
     char       line[32768];
@@ -942,6 +989,8 @@ amgtar_backup(
     int        tarpid;
     char      *file_exclude;
     char      *file_include;
+    char      *option;
+    char      *gnutar_realpath = NULL;
 
     mesgstream = fdopen(mesgf, "w");
     if (!mesgstream) {
@@ -949,9 +998,11 @@ amgtar_backup(
     }
 
     if (!gnutar_path) {
+        fprintf(mesgstream, "? GNUTAR-PATH not defined\n");
 	error(_("GNUTAR-PATH not defined"));
     }
     if (!gnutar_listdir) {
+        fprintf(mesgstream, "? GNUTAR-LISTDIR not defined\n");
 	error(_("GNUTAR-LISTDIR not defined"));
     }
 
@@ -968,16 +1019,26 @@ amgtar_backup(
         error(_("No device argument"));
     }
 
+    if (!check_exec_for_suid("GNUTAR_PATH", gnutar_path, NULL, &gnutar_realpath)) {
+	fprintf(mesgstream, "? '%s' binary is not secure\n", gnutar_path);
+	error("'%s' binary is not secure", gnutar_path);
+    }
+
+    if ((option = validate_command_options(argument))) {
+	fprintf(mesgstream, "? Invalid '%s' COMMAND-OPTIONS\n", option);
+	error("Invalid '%s' COMMAND-OPTIONS", option);
+    }
+
     qdisk = quote_string(argument->dle.disk);
 
     incrname = amgtar_get_incrname(argument,
 				   GPOINTER_TO_INT(argument->level->data),
 				   mesgstream, CMD_BACKUP);
-    cmd = stralloc(gnutar_path);
-    argv_ptr = amgtar_build_argv(argument, incrname, &file_exclude,
+    argv_ptr = amgtar_build_argv(gnutar_realpath,
+				 argument, incrname, &file_exclude,
 				 &file_include, CMD_BACKUP);
 
-    tarpid = pipespawnv(cmd, STDIN_PIPE|STDERR_PIPE, 1,
+    tarpid = pipespawnv(gnutar_realpath, STDIN_PIPE|STDERR_PIPE, 1,
 			&dumpin, &dataf, &outf, (char **)argv_ptr->pdata);
     /* close the write ends of the pipes */
 
@@ -1042,20 +1103,20 @@ amgtar_backup(
     waitpid(tarpid, &wait_status, 0);
     if (WIFSIGNALED(wait_status)) {
 	errmsg = vstrallocf(_("%s terminated with signal %d: see %s"),
-			    cmd, WTERMSIG(wait_status), dbfn());
+			    gnutar_realpath, WTERMSIG(wait_status), dbfn());
     } else if (WIFEXITED(wait_status)) {
 	if (exit_value[WEXITSTATUS(wait_status)] == 1) {
 	    errmsg = vstrallocf(_("%s exited with status %d: see %s"),
-				cmd, WEXITSTATUS(wait_status), dbfn());
+				gnutar_realpath, WEXITSTATUS(wait_status), dbfn());
 	} else {
 	    /* Normal exit */
 	}
     } else {
 	errmsg = vstrallocf(_("%s got bad exit: see %s"),
-			    cmd, dbfn());
+			    gnutar_realpath, dbfn());
     }
-    dbprintf(_("after %s %s wait\n"), cmd, qdisk);
-    dbprintf(_("amgtar: %s: pid %ld\n"), cmd, (long)tarpid);
+    dbprintf(_("after %s %s wait\n"), gnutar_realpath, qdisk);
+    dbprintf(_("amgtar: %s: pid %ld\n"), gnutar_realpath, (long)tarpid);
     if (errmsg) {
 	dbprintf("%s", errmsg);
 	g_fprintf(mesgstream, "sendbackup: error [%s]\n", errmsg);
@@ -1102,7 +1163,7 @@ amgtar_backup(
 
     amfree(incrname);
     amfree(qdisk);
-    amfree(cmd);
+    amfree(gnutar_realpath);
     g_ptr_array_free_full(argv_ptr);
 }
 
@@ -1110,21 +1171,27 @@ static void
 amgtar_restore(
     application_argument_t *argument)
 {
-    char       *cmd;
     GPtrArray  *argv_ptr = g_ptr_array_new();
     char      **env;
     int         j;
     char       *e;
     char       *include_filename = NULL;
     char       *exclude_filename = NULL;
-    int         tarpid;
+    char       *gnutar_realpath = NULL;
 
     if (!gnutar_path) {
 	error(_("GNUTAR-PATH not defined"));
     }
 
-    cmd = stralloc(gnutar_path);
-    g_ptr_array_add(argv_ptr, stralloc(gnutar_path));
+    if (!check_exec_for_suid("GNUTAR_PATH", gnutar_path, NULL, &gnutar_realpath)) {
+        error("'%s' binary is not secure", gnutar_path);
+    }
+
+    if (!security_allow_to_restore(NULL)) {
+	error("The user is not allowed to restore files");
+    }
+
+    g_ptr_array_add(argv_ptr, stralloc(gnutar_realpath));
     g_ptr_array_add(argv_ptr, stralloc("--numeric-owner"));
     if (gnutar_no_unquote)
 	g_ptr_array_add(argv_ptr, stralloc("--no-unquote"));
@@ -1267,26 +1334,11 @@ amgtar_restore(
 
     debug_executing(argv_ptr);
 
-    tarpid = fork();
-    switch (tarpid) {
-    case -1: error(_("%s: fork returned: %s"), get_pname(), strerror(errno));
-    case 0:
-	env = safe_env();
-	become_root();
-	execve(cmd, (char **)argv_ptr->pdata, env);
-	e = strerror(errno);
-	error(_("error [exec %s: %s]"), cmd, e);
-	break;
-    default: break;
-    }
-
-    waitpid(tarpid, NULL, 0);
-    if (argument->verbose == 0) {
-	if (exclude_filename)
-	    unlink(exclude_filename);
-	if (include_filename)
-	    unlink(include_filename);
-    }
+    env = safe_env();
+    become_root();
+    execve(gnutar_realpath, (char **)argv_ptr->pdata, env);
+    e = strerror(errno);
+    error(_("error [exec %s: %s]"), gnutar_realpath, e);
 }
 
 static void
@@ -1531,6 +1583,7 @@ check_no_check_device(void)
 }
 
 GPtrArray *amgtar_build_argv(
+    char *gnutar_realpath,
     application_argument_t *argument,
     char  *incrname,
     char **file_exclude,
@@ -1555,7 +1608,7 @@ GPtrArray *amgtar_build_argv(
 	dirname = argument->dle.device;
     }
 
-    g_ptr_array_add(argv_ptr, stralloc(gnutar_path));
+    g_ptr_array_add(argv_ptr, stralloc(gnutar_realpath));
 
     g_ptr_array_add(argv_ptr, stralloc("--create"));
     if (command == CMD_BACKUP && argument->dle.create_index)
@@ -1585,8 +1638,44 @@ GPtrArray *amgtar_build_argv(
 	g_ptr_array_add(argv_ptr, stralloc("--xattrs"));
     g_ptr_array_add(argv_ptr, stralloc("--listed-incremental"));
     g_ptr_array_add(argv_ptr, stralloc(incrname));
-    if (gnutar_sparse)
-	g_ptr_array_add(argv_ptr, stralloc("--sparse"));
+    if (gnutar_sparse) {
+	if (!gnutar_sparse_set) {
+	    char  *gtar_version;
+	    char  *minor_version;
+	    char  *sminor_version;
+	    char  *gv;
+	    int    major;
+	    int    minor;
+	    GPtrArray *version_ptr = g_ptr_array_new();
+
+	    g_ptr_array_add(version_ptr, gnutar_path);
+	    g_ptr_array_add(version_ptr, "--version");
+	    g_ptr_array_add(version_ptr, NULL);
+	    gtar_version = get_first_line(version_ptr);
+	    if (gtar_version) {
+		for (gv = gtar_version; *gv && !g_ascii_isdigit(*gv); gv++);
+		minor_version = index(gtar_version, '.');
+		if (minor_version) {
+		    *minor_version++ = '\0';
+		    sminor_version = index(minor_version, '.');
+		    if (sminor_version) {
+			*sminor_version = '\0';
+		    }
+		    major = atoi(gv);
+		    minor = atoi(minor_version);
+		    if (major < 1 ||
+			(major == 1 && minor < 28)) {
+			gnutar_sparse = 0;
+		    }
+		}
+	    }
+	    g_ptr_array_free(version_ptr, TRUE);
+	    amfree(gtar_version);
+	}
+	if (gnutar_sparse) {
+	    g_ptr_array_add(argv_ptr, g_strdup("--sparse"));
+	}
+    }
     if (argument->tar_blocksize) {
 	g_ptr_array_add(argv_ptr, stralloc("--blocking-factor"));
 	g_ptr_array_add(argv_ptr, stralloc(argument->tar_blocksize));
