@@ -74,11 +74,12 @@ sub new {
     $self->{'props'} = {
         'pg-cleanupwal' => 'yes',
 	'pg-max-wal-wait' => 60,
+	'pg-full-wal' => 'incr',
     };
 
     my @PROP_NAMES = qw(pg-host pg-port pg-db pg-user pg-password pg-passfile
 			psql-path pg-datadir pg-archivedir pg-cleanupwal
-			pg-max-wal-wait);
+			pg-max-wal-wait  pg-full-wal);
 
     # config is loaded by Amanda::Application (and Amanda::Script_App)
     my $conf_props = getconf($CNF_PROPERTY);
@@ -143,6 +144,8 @@ if defined $self->{'args'}->{$pdumpname};
 	$self->{'props'}->{'pg-datadir'} =  $self->{'args'}->{'device'};
     }
 
+    $self->{'props'}->{'pg-full-wal'} ||= 'incr';
+    $self->{'props'}->{'pg-full-wal'} = lc($self->{'props'}->{'pg-full-wal'});
     $self->{'args'}->{'verbose'} ||= "no";
     $self->{'args'}->{'verbose'} = string_to_boolean($self->{'args'}->{'verbose'});
 
@@ -312,11 +315,17 @@ sub command_selfcheck {
        exit(1);
    };
 
-    $self->print_to_server("disk " . quote_string($self->{args}->{disk}));
+    $self->print_to_server("disk " . quote_string($self->{args}->{disk}), $Amanda::Script_App::GOOD);
 
     $self->print_to_server("ampgsql version " . $Amanda::Constants::VERSION,
 			   $Amanda::Script_App::GOOD);
 
+    if ($self->{'props'}->{'pg-full-wal'} ne "no" &&
+	$self->{'props'}->{'pg-full-wal'} ne "full" &&
+	$self->{'props'}->{'pg-full-wal'} ne "incr") {
+	$self->print_to_server("FULL-WAL property must be set to 'NO', 'FULL' or 'INCR'",
+			       $Amanda::Script_App::ERROR);
+    }
     for my $k (keys %{$self->{'args'}}) {
         print "OK application property: $k = $self->{'args'}->{$k}\n";
     }
@@ -382,6 +391,12 @@ sub command_selfcheck {
 	    _check_parent_dirs($self->{'props'}->{'pg-archivedir'});
 	}
 
+	if (defined $self->{'props'}->{'pg-datadir'} and
+	    defined $self->{'props'}->{'pg-archivedir'}) {
+	    _check("ARCHIVEDIR", "is not inside DATADIR", "is inside DATADIR",
+	           sub {$self->{'props'}->{'pg-datadir'} ne substr($self->{'props'}->{'pg-archivedir'}, 0, length($self->{'props'}->{'pg-datadir'}))});
+	}
+
 	$try_connect &&=
 	    _check("Are both PG-PASSFILE and PG-PASSWORD set?",
 		   "No (okay)",
@@ -450,15 +465,57 @@ sub command_selfcheck {
     }
 }
 
+sub _filename_prefix {
+    my $self = shift;
+
+    my @parts = ("ampgsql", hexencode($self->{'args'}->{'host'}), hexencode($self->{'args'}->{'disk'}));
+    return $self->{'args'}->{'statedir'} . '/'  . join("-", @parts)
+}
+
 sub _state_filename {
     my ($self, $level) = @_;
 
-    my @parts = ("ampgsql", hexencode($self->{'args'}->{'host'}), hexencode($self->{'args'}->{'disk'}), $level);
-    my $statefile = $self->{'args'}->{'statedir'} . '/'  . join("-", @parts);
+    my $prefix = $self->_filename_prefix();
+    my $statefile = $prefix . "-$level";
     debug("statefile: $statefile");
     return $statefile;
 }
 
+sub _last_backup_filename {
+    my $self = shift;
+
+    my $prefix = $self->_filename_prefix();
+    my $lastfile = $prefix . "-last_backup";
+    debug("lastfile $lastfile");
+    return $lastfile
+}
+
+sub _get_last_backup {
+    my $self = shift;
+
+    my $h = new IO::File($self->_last_backup_filename(), "r");
+    $h or return undef;
+
+    my $line = <$h>;
+    $line =~ /LEVEL: (\d*)/;
+    my $level = $1;
+    $h->close();
+
+    return $level;
+}
+
+sub _write_last_backup {
+    my $self = shift;
+    my $level = shift;
+
+    my $h = new IO::File($self->_last_backup_filename(), "w");
+    $h or return undef;
+
+    debug("writing last file");
+    $h->print("LEVEL: $level\n");
+    $h->close;
+}
+
 sub _write_state_file {
     my ($self, $end_wal) = @_;
 
@@ -526,7 +583,9 @@ sub _run_tar_totals {
 
     my @cmd;
     @cmd = ($self->{'runtar'}, $self->{'args'}->{'config'},
-        $Amanda::Constants::GNUTAR, '--create', '--dereference', '--totals', @other_args);
+        $Amanda::Constants::GNUTAR, '--create', '--totals',
+				    '--blocking-factor', '64',
+				    @other_args);
     debug("running: " . join(" ", @cmd));
 
     local (*TAR_IN, *TAR_OUT, *TAR_ERR);
@@ -541,7 +600,9 @@ sub _run_tar_totals {
     while (my $l = <TAR_ERR>) {
         if ($l =~ /^Total bytes written: (\d+)/) {
             $size = $1;
-        } elsif ($l =~ /file changed as we read it/) {
+        } elsif ($l =~ /file changed as we read it/ ||
+		 $l =~ /File removed before we read it/ ||
+		 $l =~ /File shrank by \d* bytes; padding with zeros/) {
 	    chomp $l;
 	    $self->print_to_server($l, $Amanda::Script_App::GOOD);
 	    debug("TAR_ERR: $l");
@@ -754,21 +815,28 @@ sub _base_backup {
        }
        $old_die_cb->($msg);
    };
-   my $size = _run_tar_totals($self, '--file', "-",
+   my $size = _run_tar_totals($self, '--dereference', '--file', "-",
        '--directory', $self->{'props'}->{'pg-datadir'},
        '--exclude', 'postmaster.pid',
        '--exclude', 'pg_xlog/*', # contains WAL files; will be handled below
-       '--transform', "s,^,$_DATA_DIR_RESTORE/,S",
+       '--transform', "s,^./pg_tblspc,./pg_tblspc_data,S;s,^,$_DATA_DIR_RESTORE/,S",
        ".");
-   $self->{'die_cb'} = $old_die_cb;
+
+   # tar the symlink in pg_tblspc
+   $size += _run_tar_totals($self, '--file', "-",
+		'--directory', "$self->{'props'}->{'pg-datadir'}",
+		'--transform', "s,^,$_DATA_DIR_RESTORE/,S",
+                "./pg_tblspc");
 
    if ($self->{'action'} eq 'backup') {
        unless (_run_psql_command($self, "SELECT pg_stop_backup()")) {
            $self->{'die_cb'}->("Failed to call pg_stop_backup");
        }
    }
+   $self->{'die_cb'} = $old_die_cb;
 
    # determine WAL files and append and create their tar file
+   my $last_end_wal;
    my $start_wal;
    my $end_wal;
 
@@ -778,6 +846,12 @@ sub _base_backup {
 		or $self->{'die_cb'}->("A .backup file was never found in the archive "
 				    . "dir $self->{'props'}->{'pg-archivedir'}");
 	$self->_wait_for_wal($end_wal);
+	if ($self->{'props'}->{'pg-full-wal'} eq "full") {
+	    $last_end_wal = _get_prev_state($self, 0);
+	} elsif ($self->{'props'}->{'pg-full-wal'} eq "incr") {
+	    my $level = $self->_get_last_backup();
+	    $last_end_wal = _get_prev_state($self, $level);
+	}
    } else {
 	$start_wal = undef;
 	$end_wal = _get_prev_state($self, 0);
@@ -786,12 +860,14 @@ sub _base_backup {
    # now grab all of the WAL files, *inclusive* of $start_wal
    my @wal_files;
    my $adir = new IO::Dir($self->{'props'}->{'pg-archivedir'});
+   $adir or $self->{'die_cb'}->("Could not open archive WAL directory");
    while (defined(my $fname = $adir->read())) {
        if ($fname =~ /^$_WAL_FILE_PAT$/) {
            if (!defined $end_wal ||
 	       (!defined $start_wal and ($fname le $end_wal)) ||
 	       (defined $start_wal and ($fname ge $start_wal) and
-		($fname le $end_wal))) {
+		($fname le $end_wal)) ||
+	       (defined $last_end_wal and ($fname gt $last_end_wal) and ($fname le $end_wal))) {
                push @wal_files, $fname;
                debug("will store: $fname");
            } elsif (defined $start_wal and $fname lt $start_wal) {
@@ -802,20 +878,41 @@ sub _base_backup {
    $adir->close();
 
    if (@wal_files) {
-       $size += _run_tar_totals($self, '--file', "-",
+       $size += _run_tar_totals($self, '--dereference', '--file', "-",
 	   '--directory', $self->{'props'}->{'pg-archivedir'},
 	   '--transform', "s,^,$_ARCHIVE_DIR_RESTORE/,S",
 	   @wal_files);
+       #take the directory itself
+       $size += _run_tar_totals($self, '--dereference', '--file', "-",
+	   '--directory', $self->{'props'}->{'pg-archivedir'},
+	   '--transform', "s,^,$_ARCHIVE_DIR_RESTORE/,S",
+	   '--no-recursion', '.');
    } else {
        my $dummydir = $self->_make_dummy_dir_base();
-       $self->{'done_cb'}->(_run_tar_totals($self, '--file', '-',
+       $self->{'done_cb'}->(_run_tar_totals($self, '--dereference', '--file', '-',
            '--directory', $dummydir, "$_ARCHIVE_DIR_RESTORE"));
        rmtree($dummydir);
    }
 
-   $self->{'state_cb'}->($self, $end_wal);
+    $self->{'state_cb'}->($self, $end_wal);
 
-   $self->{'done_cb'}->($size);
+    # remove older WAL files.
+    # should be done only on confirmation the backup succeeded from the server
+    if (defined $start_wal || defined $last_end_wal) {
+	my $adir = new IO::Dir($self->{'props'}->{'pg-archivedir'});
+	$adir or $self->{'die_cb'}->("Could not open archive WAL directory");
+	while (defined(my $fname = $adir->read())) {
+	    if ($fname =~ /^$_WAL_FILE_PAT$/) {
+		if ((defined $start_wal and $fname lt $start_wal) ||
+		    (defined $last_end_wal and $fname le $last_end_wal)) {
+		    $self->{'unlink_cb'}->("$self->{'props'}->{'pg-archivedir'}/$fname");
+		}
+            }
+	}
+	$adir->close();
+    }
+
+    $self->{'done_cb'}->($size);
 }
 
 sub _incr_backup {
@@ -852,15 +949,18 @@ sub _incr_backup {
 
    $self->{'state_cb'}->($self, $max_wal ? $max_wal : $end_wal);
 
+   my $size;
    if (@wal_files) {
-       $self->{'done_cb'}->(_run_tar_totals($self, '--file', '-',
-           '--directory', $self->{'props'}->{'pg-archivedir'}, @wal_files));
+       $size = _run_tar_totals($self, '--dereference', '--file', '-',
+           '--directory', $self->{'props'}->{'pg-archivedir'}, @wal_files);
    } else {
        my $dummydir = $self->_make_dummy_dir();
-       $self->{'done_cb'}->(_run_tar_totals($self, '--file', '-',
-           '--directory', $dummydir, "empty-incremental"));
+       $size = _run_tar_totals($self, '--dereference', '--file', '-',
+           '--directory', $dummydir, "empty-incremental");
        rmtree($dummydir);
    }
+
+   $self->{'done_cb'}->($size);
 }
 
 sub command_backup {
@@ -894,6 +994,7 @@ sub command_backup {
    $self->{'state_cb'} = sub {
        my ($self, $end_wal) = @_;
        _write_state_file($self, $end_wal) or $self->{'die_cb'}->("Failed to write state file");
+       $self->_write_last_backup($self->{'args'}->{'level'});
    };
    my $cleanup_wal_val = $self->{'props'}->{'pg-cleanupwal'} || 'yes';
    my $cleanup_wal = string_to_boolean($cleanup_wal_val);
@@ -952,6 +1053,19 @@ sub command_restore {
 		'--ignore-zeros',
 		'--exclude', 'empty-incremental',
 		'--directory', $_ARCHIVE_DIR_RESTORE) >> 8;
+	my $uid = (stat $_ARCHIVE_DIR_RESTORE)[4];
+	if ($uid == 0) {
+	    opendir(my $dh, $_ARCHIVE_DIR_RESTORE) || die;
+	    while (my $name = readdir $dh) {
+		next if $name == '.';
+		next if $name == '..';
+		my $uid = (stat "$_ARCHIVE_DIR_RESTORE/$name")[4];
+		my $gid = (stat "$_ARCHIVE_DIR_RESTORE/$name")[5];
+		chown $uid, $gid, $_ARCHIVE_DIR_RESTORE;
+		last;
+	    }
+	    closedir $dh;
+	}
 	(0 == $status) or die("Failed to extract level $self->{'args'}->{'level'} backup (exit status: $status)");
     } else {
 	debug("extracting base of full backup to $cur_dir/$_DATA_DIR_RESTORE");
@@ -1043,6 +1157,8 @@ EOF
     exit(1);
 }
 
+my @orig_argv = @ARGV;
+
 my $opts = {};
 my $opt_version;
 
@@ -1069,6 +1185,7 @@ GetOptions(
     'cleanupwal=s',
     'archivedir=s',
     'db=s',
+    'full-wal=s',
     'host=s',
     'max-wal-wait=s',
     'passfile=s',
@@ -1085,4 +1202,6 @@ if (defined $opt_version) {
 
 my $application = Amanda::Application::ampgsql->new($opts);
 
+Amanda::Debug::debug("Arguments: " . join(' ', @orig_argv));
+
 $application->do($ARGV[0]);
