@@ -58,6 +58,7 @@
 #include "conffile.h"
 #include "getopt.h"
 #include "sendbackup.h"
+#include "security-file.h"
 
 int debug_application = 1;
 #define application_debug(i, ...) do {	\
@@ -123,9 +124,11 @@ static void amstar_estimate(application_argument_t *argument);
 static void amstar_backup(application_argument_t *argument);
 static void amstar_restore(application_argument_t *argument);
 static void amstar_validate(application_argument_t *argument);
-static GPtrArray *amstar_build_argv(application_argument_t *argument,
+static GPtrArray *amstar_build_argv(char *star_realpath,
+				application_argument_t *argument,
 				int level,
-				int command);
+				int command,
+				FILE *mesgstream);
 static int check_device(application_argument_t *argument);
 
 static char *star_path;
@@ -164,6 +167,7 @@ static struct option long_options[] = {
     {"command-options" , 1, NULL, 22},
     {"exclude-file"    , 1, NULL, 23},
     {"acl"             , 1, NULL, 24},
+    {"include-file"    , 1, NULL, 25},
     { NULL, 0, NULL, 0}
 };
 
@@ -339,6 +343,10 @@ main(
 		 else if (strcasecmp(command, "selfcheck") == 0)
 		     printf(_("ERROR [%s: bad ACL property value (%s)]\n"), get_pname(), optarg);
 		 break;
+	case 25: if (optarg)
+		     argument.dle.include_file =
+			 append_sl(argument.dle.include_file, optarg);
+		 break;
 	case ':':
 	case '?':
 		break;
@@ -390,6 +398,22 @@ main(
     return 0;
 }
 
+static char *validate_command_options(
+    application_argument_t *argument)
+{
+    GSList    *copt;
+
+    for (copt = argument->command_options; copt != NULL; copt = copt->next) {
+	char *opt = (char *)copt->data;
+
+	if (g_str_has_prefix(opt, "--compress-command")) {
+	   return opt;
+	}
+    }
+
+    return NULL;
+}
+
 static void
 amstar_support(
     application_argument_t *argument)
@@ -404,7 +428,7 @@ amstar_support(
     fprintf(stdout, "MESSAGE-LINE YES\n");
     fprintf(stdout, "MESSAGE-XML NO\n");
     fprintf(stdout, "RECORD YES\n");
-    fprintf(stdout, "INCLUDE-FILE NO\n");
+    fprintf(stdout, "INCLUDE-FILE YES\n");
     fprintf(stdout, "INCLUDE-LIST YES\n");
     fprintf(stdout, "EXCLUDE-FILE YES\n");
     fprintf(stdout, "EXCLUDE-LIST YES\n");
@@ -418,6 +442,8 @@ static void
 amstar_selfcheck(
     application_argument_t *argument)
 {
+    char *option;
+
     if (argument->dle.disk) {
 	char *qdisk = quote_string(argument->dle.disk);
 	fprintf(stdout, "OK disk %s\n", qdisk);
@@ -438,36 +464,49 @@ amstar_selfcheck(
 	amfree(qdirectory);
     }
 
-    if (argument->dle.include_list &&
-	argument->dle.include_list->nb_element >= 0) {
-	fprintf(stdout, "ERROR include-list not supported for backup\n");
+    if (((argument->dle.include_list &&
+	  argument->dle.include_list->nb_element >= 0) ||
+         (argument->dle.include_file &&
+	  argument->dle.include_file->nb_element >= 0)) &&
+	((argument->dle.exclude_list &&
+	  argument->dle.exclude_list->nb_element >= 0) ||
+         (argument->dle.exclude_file &&
+	  argument->dle.exclude_file->nb_element >= 0))) {
+	fprintf(stdout, "ERROR Can't use include and exclude simultaneously\n");
+    }
+
+    if ((option = validate_command_options(argument))) {
+	fprintf(stdout, "ERROR Invalid '%s' COMMAND-OPTIONS\n", option);
     }
 
     if (!star_path) {
 	fprintf(stdout, "ERROR STAR-PATH not defined\n");
     } else {
 	if (check_file(star_path, X_OK)) {
-	    char *star_version;
-	    GPtrArray *argv_ptr = g_ptr_array_new();
+	    char *star_realpath = NULL;
+	    if (check_exec_for_suid("STAR_PATH", star_path, stdout, &star_realpath)) {
+		char *star_version;
+		GPtrArray *argv_ptr = g_ptr_array_new();
 
-	    g_ptr_array_add(argv_ptr, star_path);
-	    g_ptr_array_add(argv_ptr, "--version");
-	    g_ptr_array_add(argv_ptr, NULL);
+		g_ptr_array_add(argv_ptr, star_realpath);
+		g_ptr_array_add(argv_ptr, "--version");
+		g_ptr_array_add(argv_ptr, NULL);
 
-	    star_version = get_first_line(argv_ptr);
+		star_version = get_first_line(argv_ptr);
 
-	    if (star_version) {
-		char *sv, *sv1;
-		for (sv = star_version; *sv && !g_ascii_isdigit(*sv); sv++);
-		for (sv1 = sv; *sv1 && *sv1 != ' '; sv1++);
-		*sv1 = '\0';
-		printf("OK amstar star-version %s\n", sv);
-	    } else {
-		printf(_("ERROR [Can't get %s version]\n"), star_path);
+		if (star_version) {
+		    char *sv, *sv1;
+		    for (sv = star_version; *sv && !g_ascii_isdigit(*sv); sv++);
+		    for (sv1 = sv; *sv1 && *sv1 != ' '; sv1++);
+		    *sv1 = '\0';
+		    printf("OK amstar star-version %s\n", sv);
+		} else {
+		    printf(_("ERROR [Can't get %s version]\n"), star_path);
+		}
+		g_ptr_array_free(argv_ptr, TRUE);
+		amfree(star_version);
 	    }
-	    g_ptr_array_free(argv_ptr, TRUE);
-	    amfree(star_version);
-
+	    amfree(star_realpath);
 	}
     }
 
@@ -496,7 +535,6 @@ amstar_estimate(
     application_argument_t *argument)
 {
     GPtrArray  *argv_ptr;
-    char       *cmd = NULL;
     int         nullfd;
     int         pipefd;
     FILE       *dumpout = NULL;
@@ -511,6 +549,8 @@ amstar_estimate(
     times_t     start_time;
     int         level = 0;
     GSList     *levels = NULL;
+    char       *option;
+    char       *star_realpath;
 
     if (!argument->level) {
 	fprintf(stderr, "ERROR No level argument\n");
@@ -534,6 +574,11 @@ amstar_estimate(
 	return;
     }
 
+    if ((option = validate_command_options(argument))) {
+	fprintf(stderr, "ERROR Invalid '%s' COMMAND-OPTIONS\n", option);
+	error("Invalid '%s' COMMAND-OPTIONS", option);
+    }
+
     qdisk = quote_string(argument->dle.disk);
     if (argument->calcsize) {
 	char *dirname;
@@ -552,13 +597,17 @@ amstar_estimate(
 	errmsg = vstrallocf(_("STAR-PATH not defined"));
 	goto common_error;
     }
-    cmd = stralloc(star_path);
+    if (!check_exec_for_suid("STAR_PATH", star_path, NULL, &star_realpath)) {
+	errmsg = g_strdup_printf("'%s' binary is not secure", star_path);
+	goto common_error;
+    }
 
     start_time = curclock();
 
     for (levels = argument->level; levels != NULL; levels = levels->next) {
 	level = GPOINTER_TO_INT(levels->data);
-	argv_ptr = amstar_build_argv(argument, level, CMD_ESTIMATE);
+	argv_ptr = amstar_build_argv(star_realpath,
+				     argument, level, CMD_ESTIMATE, NULL);
 
 	if ((nullfd = open("/dev/null", O_RDWR)) == -1) {
 	    errmsg = vstrallocf(_("Cannot access /dev/null : %s"),
@@ -566,7 +615,7 @@ amstar_estimate(
 	    goto common_error;
 	}
 
-	starpid = pipespawnv(cmd, STDERR_PIPE, 1,
+	starpid = pipespawnv(star_realpath, STDERR_PIPE, 1,
 			     &nullfd, &nullfd, &pipefd,
 			     (char **)argv_ptr->pdata);
 
@@ -609,12 +658,12 @@ amstar_estimate(
 		 walltime_str(timessub(curclock(), start_time)));
 	if(size == (off_t)-1) {
 	    errmsg = vstrallocf(_("no size line match in %s output"),
-				cmd);
+				star_realpath);
 	    dbprintf(_("%s for %s\n"), errmsg, qdisk);
 	    dbprintf(".....\n");
 	} else if(size == (off_t)0 && argument->level == 0) {
 	    dbprintf(_("possible %s problem -- is \"%s\" really empty?\n"),
-		     cmd, argument->dle.disk);
+		     star_realpath, argument->dle.disk);
 	    dbprintf(".....\n");
 	}
 	dbprintf(_("estimate size for %s level %d: %lld KB\n"),
@@ -624,22 +673,22 @@ amstar_estimate(
 
 	kill(-starpid, SIGTERM);
 
-	dbprintf(_("waiting for %s \"%s\" child\n"), cmd, qdisk);
+	dbprintf(_("waiting for %s \"%s\" child\n"), star_realpath, qdisk);
 	waitpid(starpid, &wait_status, 0);
 	if (WIFSIGNALED(wait_status)) {
 	    errmsg = vstrallocf(_("%s terminated with signal %d: see %s"),
-				cmd, WTERMSIG(wait_status), dbfn());
+				star_realpath, WTERMSIG(wait_status), dbfn());
 	} else if (WIFEXITED(wait_status)) {
 	    if (WEXITSTATUS(wait_status) != 0) {
 		errmsg = vstrallocf(_("%s exited with status %d: see %s"),
-				    cmd, WEXITSTATUS(wait_status), dbfn());
+				    star_realpath, WEXITSTATUS(wait_status), dbfn());
 	    } else {
 		/* Normal exit */
 	    }
 	} else {
-	    errmsg = vstrallocf(_("%s got bad exit: see %s"), cmd, dbfn());
+	    errmsg = vstrallocf(_("%s got bad exit: see %s"), star_realpath, dbfn());
 	}
-	dbprintf(_("after %s %s wait\n"), cmd, qdisk);
+	dbprintf(_("after %s %s wait\n"), star_realpath, qdisk);
 
 	g_ptr_array_free_full(argv_ptr);
 
@@ -649,7 +698,7 @@ amstar_estimate(
 	fprintf(stdout, "%d %lld 1\n", level, (long long)size);
     }
     amfree(qdisk);
-    amfree(cmd);
+    amfree(star_realpath);
     return;
 
 common_error:
@@ -660,7 +709,7 @@ common_error:
     fprintf(stdout, "ERROR %s\n", qerrmsg);
     amfree(errmsg);
     amfree(qerrmsg);
-    amfree(cmd);
+    amfree(star_realpath);
 }
 
 static void
@@ -668,7 +717,6 @@ amstar_backup(
     application_argument_t *argument)
 {
     int        dumpin;
-    char      *cmd = NULL;
     char      *qdisk;
     char       line[32768];
     amregex_t *rp;
@@ -691,6 +739,8 @@ amstar_backup(
     regex_t    regex_special;
     regex_t    regex_symbolic;
     regex_t    regex_hard;
+    char      *option;
+    char      *star_realpath;
 
     mesgstream = fdopen(mesgf, "w");
     if (!mesgstream) {
@@ -710,6 +760,16 @@ amstar_backup(
 	error(_("No device argument"));
     }
 
+    if (!check_exec_for_suid("STAR_PATH", star_path, NULL, &star_realpath)) {
+	fprintf(mesgstream, "? '%s' binary is not secure", star_path);
+	error("'%s' binary is not secure", star_path);
+    }
+
+    if ((option = validate_command_options(argument))) {
+	fprintf(mesgstream, "? Invalid '%s' COMMAND-OPTIONS\n", option);
+	error("Invalid '%s' COMMAND-OPTIONS\n", option);
+    }
+
     if (argument->dle.include_list &&
 	argument->dle.include_list->nb_element >= 0) {
 	fprintf(mesgstream, "? include-list not supported for backup\n");
@@ -719,11 +779,10 @@ amstar_backup(
 
     qdisk = quote_string(argument->dle.disk);
 
-    argv_ptr = amstar_build_argv(argument, level, CMD_BACKUP);
+    argv_ptr = amstar_build_argv(star_realpath,
+				 argument, level, CMD_BACKUP, mesgstream);
 
-    cmd = stralloc(star_path);
-
-    starpid = pipespawnv(cmd, STDIN_PIPE|STDERR_PIPE, 1,
+    starpid = pipespawnv(star_realpath, STDIN_PIPE|STDERR_PIPE, 1,
 			 &dumpin, &dataf, &outf, (char **)argv_ptr->pdata);
 
     g_ptr_array_free_full(argv_ptr);
@@ -822,7 +881,7 @@ amstar_backup(
     regfree(&regex_symbolic);
     regfree(&regex_hard);
 
-    dbprintf(_("gnutar: %s: pid %ld\n"), cmd, (long)starpid);
+    dbprintf(_("gnutar: %s: pid %ld\n"), star_realpath, (long)starpid);
 
     dbprintf("sendbackup: size %lld\n", (long long)dump_size);
     fprintf(mesgstream, "sendbackup: size %lld\n", (long long)dump_size);
@@ -834,26 +893,31 @@ amstar_backup(
 	fclose(indexstream);
 
     amfree(qdisk);
-    amfree(cmd);
+    amfree(star_realpath);
 }
 
 static void
 amstar_restore(
     application_argument_t *argument)
 {
-    char       *cmd;
     GPtrArray  *argv_ptr = g_ptr_array_new();
     char      **env;
     int         j;
     char       *e;
+    char       *star_realpath;
 
     if (!star_path) {
 	error(_("STAR-PATH not defined"));
     }
+    if (!check_exec_for_suid("STAR_PATH", star_path, NULL, &star_realpath)) {
+	error("'%s' binary is not secure", star_path);
+    }
 
-    cmd = stralloc(star_path);
+    if (!security_allow_to_restore(NULL)) {
+        error("The user is not allowed to restore files");
+    }
 
-    g_ptr_array_add(argv_ptr, stralloc(star_path));
+    g_ptr_array_add(argv_ptr, stralloc(star_realpath));
     if (star_directory) {
 	struct stat stat_buf;
 	if(stat(star_directory, &stat_buf) != 0) {
@@ -912,9 +976,9 @@ amstar_restore(
     debug_executing(argv_ptr);
     env = safe_env();
     become_root();
-    execve(cmd, (char **)argv_ptr->pdata, env);
+    execve(star_realpath, (char **)argv_ptr->pdata, env);
     e = strerror(errno);
-    error(_("error [exec %s: %s]"), cmd, e);
+    error(_("error [exec %s: %s]"), star_realpath, e);
 
 }
 
@@ -955,9 +1019,11 @@ pipe_to_null:
 }
 
 static GPtrArray *amstar_build_argv(
+    char *star_realpath,
     application_argument_t *argument,
     int   level,
-    int   command)
+    int   command,
+    FILE *mesgstream)
 {
     char      *dirname;
     char      *fsname;
@@ -987,7 +1053,7 @@ static GPtrArray *amstar_build_argv(
 	tardumpfile = stralloc(star_tardumps);
     }
 
-    g_ptr_array_add(argv_ptr, stralloc(star_path));
+    g_ptr_array_add(argv_ptr, stralloc(star_realpath));
 
     g_ptr_array_add(argv_ptr, stralloc("-c"));
     g_ptr_array_add(argv_ptr, stralloc("-f"));
@@ -1029,6 +1095,24 @@ static GPtrArray *amstar_build_argv(
     if (command == CMD_BACKUP && argument->dle.create_index)
 	g_ptr_array_add(argv_ptr, stralloc("-v"));
 
+    if (((argument->dle.include_file &&
+	  argument->dle.include_file->nb_element >= 1) ||
+	 (argument->dle.include_list &&
+	  argument->dle.include_list->nb_element >= 1)) &&
+	((argument->dle.exclude_file &&
+	  argument->dle.exclude_file->nb_element >= 1) ||
+	 (argument->dle.exclude_list &&
+	  argument->dle.exclude_list->nb_element >= 1))) {
+
+	if (mesgstream && command == CMD_BACKUP) {
+	    fprintf(mesgstream, "? include and exclude specified, disabling exclude\n");
+	}
+	free_sl(argument->dle.exclude_file);
+	argument->dle.exclude_file = NULL;
+	free_sl(argument->dle.exclude_list);
+	argument->dle.exclude_list = NULL;
+    }
+
     if ((argument->dle.exclude_file &&
 	 argument->dle.exclude_file->nb_element >= 1) ||
 	(argument->dle.exclude_list &&
@@ -1077,6 +1161,47 @@ static GPtrArray *amstar_build_argv(
 	}
     }
 
+    if (argument->dle.include_file &&
+	argument->dle.include_file->nb_element >= 1) {
+	sle_t *excl;
+	for (excl = argument->dle.include_file->first; excl != NULL;
+	     excl = excl->next) {
+	    char *ex;
+	    if (g_str_equal(excl->name, "./")) {
+		ex = g_strdup_printf("pat=%s", excl->name+2);
+	    } else {
+		ex = g_strdup_printf("pat=%s", excl->name);
+	    }
+	    g_ptr_array_add(argv_ptr, ex);
+	}
+    }
+    if (argument->dle.include_list &&
+	argument->dle.include_list->nb_element >= 1) {
+	sle_t *excl;
+	for (excl = argument->dle.include_list->first; excl != NULL;
+	     excl = excl->next) {
+	    char *exclname = fixup_relative(excl->name, argument->dle.device);
+	    FILE *include;
+	    char *aexc;
+	    if ((include = fopen(exclname, "r")) != NULL) {
+		while ((aexc = agets(include)) != NULL) {
+		    if (aexc[0] != '\0') {
+			char *ex;
+			if (g_str_equal(aexc, "./")) {
+			    ex = g_strdup_printf("pat=%s", aexc+2);
+			} else {
+			    ex = g_strdup_printf("pat=%s", aexc);
+			}
+			g_ptr_array_add(argv_ptr, ex);
+		    }
+		    amfree(aexc);
+		}
+		fclose(include);
+	    }
+	    amfree(exclname);
+	}
+    }
+
     /* It is best to place command_options at the and of command line.
      * For example '-find' option requires that it is the last option used.
      * See: http://cdrecord.berlios.de/private/man/star/star.1.html
