@@ -293,7 +293,7 @@ sub estimate_snapshot
     }
     if ($level == 0) {
 	my $compratio = $self->get_compratio();
-	chop($compratio);
+	$compratio =~ s/x$//;
 	$msg *= $compratio;
     }
 
@@ -312,7 +312,9 @@ sub get_compratio
     $pid = open3($wtr, $rdr, $err, $cmd);
     close $wtr;
     my ($msg) = <$rdr>;
+    chomp($msg) if defined $msg;
     my ($errmsg) = <$err>;
+    chomp($errmsg) if defined $errmsg;
     waitpid $pid, 0;
     close $rdr;
     close $err;
@@ -466,6 +468,8 @@ my @opt_exclude_list;
 my @opt_include_list;
 my $opt_directory;
 
+my @orig_argv = @ARGV;
+
 Getopt::Long::Configure(qw{bundling});
 GetOptions(
     'config=s'        => \$opt_config,
@@ -489,5 +493,7 @@ GetOptions(
 
 my $application = Amanda::Application::Amzfs_sendrecv->new($opt_config, $opt_host, $opt_disk, $opt_device, \@opt_level, $opt_index, $opt_message, $opt_collection, $opt_record, $df_path, $zfs_path, $pfexec_path, $pfexec, $opt_keep_snapshot, \@opt_exclude_list, \@opt_include_list, $opt_directory);
 
+Amanda::Debug::debug("Arguments: " . join(' ', @orig_argv));
+
 $application->do($ARGV[0]);
 
