@@ -37,6 +37,7 @@
 #include "amanda.h"
 #include "util.h"
 #include "conffile.h"
+#include "client_util.h"
 
 int main(int argc, char **argv);
 
@@ -50,7 +51,9 @@ main(
     char *e;
     char *dbf;
     char *cmdline;
+    char *my_realpath = NULL;
 #endif
+    int good_option;
 
     if (argc > 1 && argv && argv[1] && g_str_equal(argv[1], "--version")) {
 	printf("runtar-%s\n", VERSION);
@@ -140,10 +143,57 @@ main(
     argc--;
     argv++;
 
-    cmdline = stralloc(GNUTAR);
+    if (!check_exec_for_suid("GNUTAR_PATH", GNUTAR, stderr, &my_realpath)) {
+	dbclose();
+	exit(1);
+    }
+
+    cmdline = stralloc(my_realpath);
+    good_option = 0;
     for (i = 1; argv[i]; i++) {
 	char *quoted;
 
+	if (good_option <= 0) {
+	    if (g_str_has_prefix(argv[i],"--rsh-command") ||
+		g_str_has_prefix(argv[i],"--to-command") ||
+		g_str_has_prefix(argv[i],"--info-script") ||
+		g_str_has_prefix(argv[i],"--new-volume-script") ||
+		g_str_has_prefix(argv[i],"--rmt-command") ||
+		g_str_has_prefix(argv[i],"--use-compress-program")) {
+		/* Filter potential malicious option */
+		good_option = 0;
+	    } else if (g_str_has_prefix(argv[i],"--create") ||
+		g_str_has_prefix(argv[i],"--totals") ||
+		g_str_has_prefix(argv[i],"--dereference") ||
+		g_str_has_prefix(argv[i],"--no-recursion") ||
+		g_str_has_prefix(argv[i],"--one-file-system") ||
+		g_str_has_prefix(argv[i],"--incremental") ||
+		g_str_has_prefix(argv[i],"--atime-preserve") ||
+		g_str_has_prefix(argv[i],"--sparse") ||
+		g_str_has_prefix(argv[i],"--ignore-failed-read") ||
+		g_str_has_prefix(argv[i],"--numeric-owner")) {
+		/* Accept theses options */
+		good_option++;
+	    } else if (g_str_has_prefix(argv[i],"--blocking-factor") ||
+		g_str_has_prefix(argv[i],"--file") ||
+		g_str_has_prefix(argv[i],"--directory") ||
+		g_str_has_prefix(argv[i],"--exclude") ||
+		g_str_has_prefix(argv[i],"--transform") ||
+		g_str_has_prefix(argv[i],"--listed-incremental") ||
+		g_str_has_prefix(argv[i],"--newer") ||
+		g_str_has_prefix(argv[i],"--exclude-from") ||
+		g_str_has_prefix(argv[i],"--files-from")) {
+		/* Accept theses options with the following argument */
+		good_option += 2;
+	    } else if (argv[i][0] != '-') {
+		good_option++;
+	    }
+	}
+	if (good_option <= 0) {
+	    error("error [%s invalid option: %s]", get_pname(), argv[i]);
+	}
+	good_option--;
+
 	quoted = quote_string(argv[i]);
 	cmdline = vstrextend(&cmdline, " ", quoted, NULL);
 	amfree(quoted);
@@ -157,15 +207,15 @@ main(
     }
     dbclose();
 
-    execve(GNUTAR, argv, safe_env());
+    execve(my_realpath, argv, safe_env());
 
     e = strerror(errno);
     dbreopen(dbf, "more");
     amfree(dbf);
-    dbprintf(_("execve of %s failed (%s)\n"), GNUTAR, e);
+    dbprintf(_("execve of %s failed (%s)\n"), my_realpath, e);
     dbclose();
 
-    g_fprintf(stderr, _("runtar: could not exec %s: %s\n"), GNUTAR, e);
+    g_fprintf(stderr, _("runtar: could not exec %s: %s\n"), my_realpath, e);
     return 1;
 #endif
 }
