@@ -44,6 +44,7 @@
 #include "amandad.h"
 #include "amxml.h"
 #include "base64.h"
+#include "security-file.h"
 
 #ifdef SAMBA_CLIENT
 #include "findpass.h"
@@ -348,6 +349,8 @@ main(
     }
 
 checkoverall:
+    check_security_file_permission(stdout);
+
     check_overall();
 
     amfree(line);
@@ -994,6 +997,16 @@ check_overall(void)
 
     if( need_runtar )
     {
+#ifdef GNUTAR
+	char *my_pname = g_strdup(get_pname());
+	char *gnutar_realpath = NULL;
+
+	set_pname("runtar");
+	check_exec_for_suid("GNUTAR_PATH", GNUTAR, stdout, &gnutar_realpath);
+	set_pname(my_pname);
+	amfree(gnutar_realpath);
+#endif
+
 	cmd = vstralloc(amlibexecdir, "/", "runtar", NULL);
 	check_file(cmd,X_OK);
 	check_suid(cmd);
@@ -1314,9 +1327,16 @@ print_platform(void)
 		}
 		fclose(release);
 	    }
-	    amfree(uname);
-	} else {
-	    amfree(uname);
+	} else if (uname && strlen(uname) >= 3 &&
+		   g_strcasecmp(uname+strlen(uname)-3, "bsd") == 0) {
+	    distro = g_strdup(uname);
+	    argv_ptr = g_ptr_array_new();
+            g_ptr_array_add(argv_ptr, UNAME_PATH);
+            g_ptr_array_add(argv_ptr, "-r");
+            g_ptr_array_add(argv_ptr, NULL);
+            platform = get_first_line(argv_ptr);
+            g_ptr_array_free(argv_ptr, TRUE);
+	} else if (!stat("/usr/bin/sw_vers", &stat_buf)) {
 	    argv_ptr = g_ptr_array_new();
 	    g_ptr_array_add(argv_ptr, "/usr/bin/sw_vers");
 	    g_ptr_array_add(argv_ptr, "-productName");
@@ -1336,6 +1356,7 @@ print_platform(void)
 		platform = g_strdup_printf("%s %s", productVersion, productVersion);
 	    }
 	}
+	amfree(uname);
 
     }
 
