@@ -42,6 +42,10 @@
 
 #include "amflock.h"
 
+#define GCC_VERSION (__GNUC__ * 10000 \
+                     + __GNUC_MINOR__ * 100 \
+                     + __GNUC_PATCHLEVEL__)
+
 /*
  * Force large file source even if configure guesses wrong.
  */
@@ -686,7 +690,7 @@ time_t	unctime(char *timestr);
 
 /* from old bsd-security.c */
 extern int debug;
-extern int check_security(sockaddr_union *, char *, unsigned long, char **);
+extern int check_security(sockaddr_union *, char *, unsigned long, char **, char *);
 
 /*
  * Handle functions which are not always declared on all systems.  This
