@@ -57,13 +57,19 @@ $ENV{'GNUPGHOME'} = "$AMANDA_HOME/.gnupg";
 sub do_gpg_agent() {
     my $path=`which gpg-agent 2>/dev/null`;
     chomp $path;
+
     if ($path =~ /^no gpg-agent/) {
 	return "";
     }
 
-    if (-x $path) {
-	return "gpg-agent --daemon --";
+    if (!-x $path) {
+	return "";
     }
+
+    if (system("$path --use-standard-socket-p 2>/dev/null")) {
+	return "$path --daemon --";
+    }
+
     return ""
 }
 
@@ -83,6 +89,7 @@ sub encrypt() {
     my $gpg_agent_cmd = do_gpg_agent();
     my $gpg = which_gpg();
     system "$gpg_agent_cmd $gpg --batch -z 0 --no-secmem-warning --disable-mdc --symmetric --cipher-algo AES256 --passphrase-fd 3  3<$AM_PASS";
+    sleep(2); # allow gpg-agent the time to exit
     if ($? == -1) {
 	print STDERR "failed to execute gpg: $!\n";
 	exit (1);
@@ -99,6 +106,7 @@ sub decrypt() {
     my $gpg_agent_cmd = do_gpg_agent();
     my $gpg = which_gpg();
     system "$gpg_agent_cmd $gpg --batch --quiet --no-mdc-warning --decrypt --passphrase-fd 3  3<$AM_PASS";
+    sleep(2); # allow gpg-agent the time to exit
     if ($? == -1) {
 	print STDERR "failed to execute gpg: $!\n";
 	exit (1);
