@@ -119,8 +119,12 @@ my %build_info = (
     'vxdump' => $Amanda::Constants::VXDUMP,
     'vxrestore' => $Amanda::Constants::VXRESTORE,
     'samba_client' => $Amanda::Constants::SAMBA_CLIENT,
+    'bsdtar' => $Amanda::Constants::BSDTAR,
     'gnutar' => $Amanda::Constants::GNUTAR,
     'star' => $Amanda::Constants::STAR,
+    'bsdtar_path' => $Amanda::Constants::BSDTAR,
+    'gnutar_path' => $Amanda::Constants::GNUTAR,
+    'star_path' => $Amanda::Constants::STAR,
     'compress_path' => $Amanda::Constants::COMPRESS_PATH,
     'uncompress_path' => $Amanda::Constants::UNCOMPRESS_PATH,
     'aix_backup' => $Amanda::Constants::AIX_BACKUP,
