@@ -1404,6 +1404,10 @@ void set_config_overrides(config_overrides_t *co);
  * Initialization
  */
 
+/* If the config is initialized */
+gboolean
+config_is_initialized(void);
+
 /* Constants for config_init */
 typedef enum {
     /* Use arg_config_name, if not NULL */
