@@ -329,7 +329,8 @@ interruptible_accept(
     struct sockaddr *addr,
     socklen_t *addrlen,
     gboolean (*prolong)(gpointer data),
-    gpointer prolong_data)
+    gpointer prolong_data,
+    time_t timeout)
 {
     SELECT_ARG_TYPE readset;
     struct timeval tv;
@@ -343,17 +344,21 @@ interruptible_accept(
     memset(&readset, 0, SIZEOF(readset));
 
     while (1) {
-	if (!prolong(prolong_data)) {
+	if (prolong && !prolong(prolong_data)) {
 	    errno = 0;
 	    return -1;
 	}
+	if (time(NULL) > timeout) {
+	    errno = ETIMEDOUT;
+	    return -1;
+	}
 
 	FD_ZERO(&readset);
 	FD_SET(sock, &readset);
 
 	/* try accepting for 1s */
 	memset(&tv, 0, SIZEOF(tv));
-    	tv.tv_sec = 1;
+	tv.tv_sec = 1;
 
 	nfound = select(sock+1, &readset, NULL, NULL, &tv);
 	if (nfound < 0) {
@@ -1076,7 +1081,7 @@ int copy_file(
     if ((outfd = open(dst, O_WRONLY|O_CREAT, 0600)) == -1) {
 	save_errno = errno;
 	quoted = quote_string(dst);
-	*errmsg = vstrallocf(_("Can't open file '%s' for writting: %s"),
+	*errmsg = vstrallocf(_("Can't open file '%s' for writing: %s"),
 			    quoted, strerror(save_errno));
 	amfree(quoted);
 	close(infd);
@@ -1414,7 +1419,7 @@ check_running_as(running_as_flags who)
     }
 
     if (uid_me != uid_target) {
-	error(_("running as user \"%s\" instead of \"%s\""), uname_me, uname_target);
+	error("must be executed as the \"%s\" user instead of the \"%s\" user", uname_target, uname_me);
 	/*NOTREACHED*/
     }
     amfree(uname_me);
@@ -1617,7 +1622,7 @@ debug_executing(
 	cmdline = vstrextend(&cmdline, " ", arg, NULL);
 	amfree(arg);
     }
-    g_debug("Executing: %s\n", cmdline);
+    g_debug("Executing: %s", cmdline);
     amfree(cmdline);
 }
 
@@ -1720,3 +1725,207 @@ make_amanda_tmpdir(void)
 	return TRUE;
     }
 }
+
+#define POLY 0x82F63B78
+#if defined __GNUC__ && GCC_VERSION > 40300 && (defined __x86_64__ || defined __i386__ || defined __i486__ || defined __i586__ || defined __i686__)
+static int get_sse42(void)
+{
+    uint32_t op, eax, ebx, ecx, edx;
+    op = 1;
+#ifdef __i386__
+    __asm__ volatile(
+		"pushl %%ebx;\n\t"
+		"cpuid;\n\t"
+		"movl %%ebx, %1;\n\t"
+		"popl %%ebx;\n\t"
+                : "=a" (eax), "=r" (ebx), "=c" (ecx), "=d" (edx)
+		: "a" (op)
+		: "cc"
+    );
+#else
+    __asm__ volatile(
+		"cpuid;\n\t"
+                : "=a" (eax), "=b" (ebx), "=c" (ecx), "=d" (edx)
+		: "a" (op)
+		: "cc"
+    );
+#endif
+    return (ecx >> 20) & 1;
+}
+#endif
+
+static uint32_t crc_table[16][256];
+static gboolean crc_initialized = FALSE;
+int have_sse42 = 0;
+void (* crc32_function)(uint8_t *buf, size_t len, crc_t *crc);
+
+#if defined __GNUC__ && GCC_VERSION > 40300 && (defined __x86_64__ || defined __i386__ || defined __i486__ || defined __i586__ || defined __i686__)
+  #include "amcrc32chw.h"
+#endif
+
+/* Run this function previously */
+void
+make_crc_table(void)
+{
+    int i;
+    int j;
+    int slice;
+
+    if (!crc_initialized) {
+#if defined __GNUC__ && GCC_VERSION > 40300 && (defined __x86_64__ || defined __i386__ || defined __i486__ || defined __i586__ || defined __i686__)
+	have_sse42 = get_sse42();
+	if (have_sse42) {
+	    crc32c_init_hw();
+	    crc32_function = &crc32c_add_hw;
+	} else
+#endif
+            crc32_function = &crc32_add_16bytes;
+
+	for (i = 0; i < 256; i++) {
+	    uint32_t c = i;
+	    for (j = 0; j < 8; j++) {
+		c = (c & 1) ? (POLY ^ (c >> 1)) : (c >> 1);
+	    }
+	    crc_table[0][i] = c;
+	}
+	for (i = 0; i < 256; i++) {
+	    for (slice = 1; slice < 16; slice++) {
+		crc_table[slice][i] = (crc_table[slice - 1][i] >> 8) ^ crc_table[0][crc_table[slice - 1][i] & 0xFF];
+	    }
+
+	}
+	crc_initialized = TRUE;
+    }
+}
+
+void
+crc32_init(
+    crc_t *crc)
+{
+    make_crc_table();
+    crc->crc = 0xFFFFFFFF;
+    crc->size = 0;
+}
+
+#ifdef __GNUC__
+  #define PREFETCH(location) __builtin_prefetch(location)
+#else
+  #define PREFETCH(location)
+#endif
+
+void crc32_add_1byte(uint8_t *buf, size_t len, crc_t *crc);
+void crc32_add_16bytes(uint8_t *buf, size_t len, crc_t *crc);
+
+void
+crc32_add_1byte(
+    uint8_t *buf,
+    size_t len,
+    crc_t *crc)
+{
+    crc->size += len;
+    while (len-- > 0) {
+       crc->crc = crc_table[0][(crc->crc ^ *buf++) & 0xFF] ^ (crc->crc >> 8);
+     }
+}
+
+#ifdef WORDS_BIGENDIAN
+static inline uint32_t swap(uint32_t data);
+static inline uint32_t
+swap(
+    uint32_t data)
+{
+#if defined(__GNUC__) && GCC_VERSION >= 40300
+    return __builtin_bswap32(data);
+#else
+    return (data >> 24) |
+          ((data >> 8) & 0x0000FF00) |
+          ((data << 8) & 0x00FF0000) |
+           (data << 24);
+#endif
+}
+#endif
+
+void
+crc32_add_16bytes(
+    uint8_t *buf,
+    size_t len,
+    crc_t *crc)
+{
+    uint32_t *buf32 = (uint32_t *)buf;
+    size_t i;
+    crc->size += len;
+
+    while (len >= 256) {
+	PREFETCH(((const char*) buf) + 256);
+	for (i = 0; i < 4; i++) {
+#ifdef WORDS_BIGENDIAN
+            uint32_t one   = *buf32++ ^ swap(crc->crc);
+            uint32_t two   = *buf32++;
+            uint32_t three = *buf32++;
+            uint32_t four  = *buf32++;
+            crc->crc  = crc_table[ 0][ four         & 0xFF] ^
+                        crc_table[ 1][(four  >>  8) & 0xFF] ^
+                        crc_table[ 2][(four  >> 16) & 0xFF] ^
+                        crc_table[ 3][(four  >> 24) & 0xFF] ^
+                        crc_table[ 4][ three        & 0xFF] ^
+                        crc_table[ 5][(three >>  8) & 0xFF] ^
+                        crc_table[ 6][(three >> 16) & 0xFF] ^
+                        crc_table[ 7][(three >> 24) & 0xFF] ^
+                        crc_table[ 8][ two          & 0xFF] ^
+                        crc_table[ 9][(two   >>  8) & 0xFF] ^
+                        crc_table[10][(two   >> 16) & 0xFF] ^
+                        crc_table[11][(two   >> 24) & 0xFF] ^
+                        crc_table[12][ one          & 0xFF] ^
+                        crc_table[13][(one   >>  8) & 0xFF] ^
+                        crc_table[14][(one   >> 16) & 0xFF] ^
+                        crc_table[15][(one   >> 24) & 0xFF];
+
+#else
+	    uint32_t one   = *buf32++ ^ crc->crc;
+	    uint32_t two   = *buf32++;
+	    uint32_t three = *buf32++;
+	    uint32_t four  = *buf32++;
+	    crc->crc  = crc_table[ 0][(four  >> 24) & 0xFF] ^
+		        crc_table[ 1][(four  >> 16) & 0xFF] ^
+		        crc_table[ 2][(four  >>  8) & 0xFF] ^
+		        crc_table[ 3][ four         & 0xFF] ^
+		        crc_table[ 4][(three >> 24) & 0xFF] ^
+		        crc_table[ 5][(three >> 16) & 0xFF] ^
+		        crc_table[ 6][(three >>  8) & 0xFF] ^
+		        crc_table[ 7][ three        & 0xFF] ^
+		        crc_table[ 8][(two   >> 24) & 0xFF] ^
+		        crc_table[ 9][(two   >> 16) & 0xFF] ^
+		        crc_table[10][(two   >>  8) & 0xFF] ^
+		        crc_table[11][ two          & 0xFF] ^
+		        crc_table[12][(one   >> 24) & 0xFF] ^
+		        crc_table[13][(one   >> 16) & 0xFF] ^
+		        crc_table[14][(one   >>  8) & 0xFF] ^
+		        crc_table[15][ one          & 0xFF];
+#endif
+	}
+	len -= 4*16;
+    }
+    buf = (uint8_t *)buf32;
+
+    while (len-- > 0) {
+	crc->crc = crc_table[0][(crc->crc ^ *buf++) & 0xFF] ^ (crc->crc >> 8);
+    }
+}
+
+void
+crc32_add(
+    uint8_t *buf,
+    size_t len,
+    crc_t *crc)
+{
+    crc32_function(buf, len, crc);
+    return;
+}
+
+uint32_t
+crc32_finish(
+    crc_t *crc)
+{
+    return crc->crc ^ 0xFFFFFFFF;
+}
+
