@@ -324,7 +324,7 @@ sub write_to_port {
 $handle = "11-11111";
 $datestamp = "20070102030405";
 run_taper(4096, "single-part and multipart FILE-WRITE");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(1024*1024, "localhost", "/home");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /home 0 $datestamp \"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" 12");
@@ -429,7 +429,7 @@ check_logs([
 $handle = '11-22223';
 $datestamp = "20070102030406";
 run_taper(4096, "multipart partial FILE-WRITE");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(1024*1024, "localhost", "/usr", 1);
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /usr 0 $datestamp 524288 \"\" \"\" 1 \"\" \"\" \"\" \"\" 512");
@@ -467,7 +467,7 @@ check_logs([
 $handle = "11-33333";
 $datestamp = "19780615010203";
 run_taper(4096, "multipart PORT-WRITE");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("PORT-WRITE worker0 $handle localhost /var 0 $datestamp 524288 \"\" 393216 1 0 \"\" \"\" 0 AMANDA");
 like(taper_reply, qr/^PORT worker0 $handle (\d+) "?(\d+\.\d+\.\d+\.\d+:\d+;?)+"?$/,
@@ -519,7 +519,7 @@ check_logs([
 $handle = "11-44444";
 $datestamp = "19411207000000";
 run_taper(4096, "testing NO-NEW-TAPE from the driver on 1st request");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(1024*1024, "localhost", "/home");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /home 0 $datestamp 0 \"\" 0 1 0 \"\" \"\" 0 912");
@@ -543,7 +543,7 @@ check_logs([
 $handle = "11-55555";
 $datestamp = "19750711095836";
 run_taper(1024, "PORT-WRITE retry on EOT (mem cache)");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("PORT-WRITE worker0 $handle localhost /usr/local 0 $datestamp 786432 \"\" 786432 1 0 \"\" \"\" 0 AMANDA");
 like(taper_reply, qr/^PORT worker0 $handle (\d+) "?(\d+\.\d+\.\d+\.\d+:\d+;?)+"?$/,
@@ -597,7 +597,7 @@ check_logs([
 $handle = "11-66666";
 $datestamp = "19470815000000";
 run_taper(1024, "FILE-WRITE retry on EOT");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(1575936, "localhost", "/usr");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /usr 0 $datestamp \"\" \"\" \"\" 1 786432 \"\" \"\" \"\" 1112");
@@ -646,7 +646,7 @@ check_logs([
 $handle = "11-77777";
 $datestamp = "20090427212500";
 run_taper(1024, "PORT-WRITE retry on EOT (disk cache)");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("PORT-WRITE worker0 $handle localhost /usr/local 0 $datestamp 786432 \"$Installcheck::TMP\" 786432 1 0 \"\" \"\" 0 AMANDA");
 like(taper_reply, qr/^PORT worker0 $handle (\d+) "?(\d+\.\d+\.\d+\.\d+:\d+;?)+"?$/,
@@ -701,7 +701,7 @@ check_logs([
 $handle = "11-88888";
 $datestamp = "20090424173000";
 run_taper(1024, "PORT-WRITE failure on EOT (no cache)");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("PORT-WRITE worker0 $handle localhost /var/log 0 $datestamp 0 \"\" 0 0 0 \"\" \"\" 0 AMANDA");
 like(taper_reply, qr/^PORT worker0 $handle (\d+) "?(\d+\.\d+\.\d+\.\d+:\d+;?)+"?$/,
@@ -758,7 +758,7 @@ check_logs([
 $handle = "11-99999";
 $datestamp = "20100101000000";
 run_taper(512, "FILE-WRITE runs out of tapes");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(512*1024, "localhost", "/music");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /music 0 $datestamp \"\" \"\" \"\" 1 262144 \"none\" \"\" 10240 1312");
@@ -796,7 +796,7 @@ check_logs([
 $handle = "22-00000";
 $datestamp = "20200202222222";
 run_taper(4096, "multipart PORT-WRITE");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("PORT-WRITE worker0 $handle localhost /sbin 0 $datestamp 999999 \"\" 655360 1 \"\" \"\" \"\" \"\" AMANDA");
 like(taper_reply, qr/^PORT worker0 $handle (\d+) "?(\d+\.\d+\.\d+\.\d+:\d+;?)+"?$/,
@@ -842,7 +842,7 @@ check_logs([
 $handle = "33-11111";
 $datestamp = "20090101010000";
 run_taper(1024, "first in a sequence");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(500000, "localhost", "/u01");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /u01 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1412");
@@ -902,7 +902,7 @@ cleanup_log();
 $handle = "33-33333";
 $datestamp = "20090202020000";
 run_taper(1024, "second in a sequence", keep_config => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(300000, "localhost", "/u01");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /u01 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1612");
@@ -966,7 +966,7 @@ cleanup_log();
 $handle = "33-55555";
 $datestamp = "20090303030000";
 run_taper(1024, "failure to overwrite a volume", keep_config => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(32768, "localhost", "/u03");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /u03 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1812");
@@ -1010,10 +1010,10 @@ my $handle0 = "66-00000";
 my $handle1 = "66-11111";
 $datestamp = "20090202020000";
 run_taper(1024, "with 2 workers", new_vtapes => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("START-TAPER worker1 $datestamp");
-like(taper_reply, qr/^TAPER-OK worker1$/,
+like(taper_reply, qr/^TAPER-OK worker1 ALLOW-TAKE-SCRIBE-FROM$/,
        "got TAPER-OK") or die;
 make_holding_file(300000, "localhost", "/u01");
 taper_cmd("FILE-WRITE worker0 $handle0 \"$test_filename\" localhost /u01 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1612");
@@ -1072,10 +1072,10 @@ cleanup_log();
 $handle = "66-22222";
 $datestamp = "20090202020000";
 run_taper(1024, "with 2 workers and a take_scribe", new_vtapes => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("START-TAPER worker1 $datestamp");
-like(taper_reply, qr/^TAPER-OK worker1$/,
+like(taper_reply, qr/^TAPER-OK worker1 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(1000000, "localhost", "/u01");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /u01 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1612");
@@ -1140,7 +1140,7 @@ SKIP : {
     $datestamp = "19780615010305";
     run_taper(4096, "multipart directtcp PORT-WRITE",
 	ndmp_server => $ndmp);
-    like(taper_reply, qr/^TAPER-OK worker0$/,
+    like(taper_reply, qr/^TAPER-OK worker0 NO-TAKE-SCRIBE-FROM$/,
 	    "got TAPER-OK") or die;
     # note that, with the addition of the new splitting params, this does the "sensible"
     # thing and uses the tape_splitsize, not the fallback_splitsize (this is a change from
@@ -1215,7 +1215,7 @@ SKIP : {
     $datestamp = "19780615010305";
     run_taper(4096, "multipart directtcp PORT-WRITE, with a zero-byte part",
 	ndmp_server => $ndmp);
-    like(taper_reply, qr/^TAPER-OK worker0$/,
+    like(taper_reply, qr/^TAPER-OK worker0 NO-TAKE-SCRIBE-FROM$/,
 	    "got TAPER-OK") or die;
     # use a different part size this time, to hit EOM "on the head"
     taper_cmd("PORT-WRITE worker0 $handle localhost /var 0 $datestamp 425984 \"\" 327680 \"\" \"\" \"\" \"\" \"\" DIRECTTCP");
@@ -1277,7 +1277,7 @@ $handle = "77-11111";
 $datestamp = "20090302020000";
 run_taper(1024, "without LEOM and without allow-split", new_vtapes => 1);
 make_holding_file(1024*1024, "localhost", "/usr");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /usr 0 $datestamp 262144 \"\" \"\" \"0\" \"\" \"\" \"\" \"\" 1612");
 like(taper_reply, qr/^REQUEST-NEW-TAPE $handle$/,
@@ -1307,7 +1307,7 @@ $handle = "77-11112";
 $datestamp = "20090303020000";
 run_taper(1024, "with LEOM and without allow-split", new_vtapes => 1, leom => 1);
 make_holding_file(1024*1024, "localhost", "/usr");
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /usr 0 $datestamp 262144 \"\" \"\" \"0\" \"\" \"\" \"\" \"\" 1612");
 like(taper_reply, qr/^REQUEST-NEW-TAPE $handle$/,
@@ -1340,7 +1340,7 @@ cleanup_log();
 $handle = "11-11111";
 $datestamp = "20070102030405";
 run_taper(4096, "single-part and multipart FILE-WRITE", taperscan => "lexical", new_vtapes => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(1024*1024, "localhost", "/home");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /home 0 $datestamp \"\" \"\" \"\" \"\" \"\" \"\" \"\" \"\" 12");
@@ -1393,7 +1393,7 @@ check_logs([
 $handle = "33-11111";
 $datestamp = "20090101010000";
 run_taper(1024, "first in a sequence", taperscan => "lexical", new_vtapes => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(500000, "localhost", "/u01");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /u01 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1412");
@@ -1453,7 +1453,7 @@ cleanup_log();
 $handle = "33-33333";
 $datestamp = "20090202020000";
 run_taper(1024, "second in a sequence", keep_config => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 make_holding_file(300000, "localhost", "/u01");
 taper_cmd("FILE-WRITE worker0 $handle \"$test_filename\" localhost /u01 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1612");
@@ -1525,10 +1525,10 @@ $handle0 = "66-00000";
 $handle1 = "66-11111";
 $datestamp = "20090202020000";
 run_taper(1024, "with 2 workers", new_vtapes => 1, taperscan => "lexical", new_vtapes => 1);
-like(taper_reply, qr/^TAPER-OK worker0$/,
+like(taper_reply, qr/^TAPER-OK worker0 ALLOW-TAKE-SCRIBE-FROM$/,
 	"got TAPER-OK") or die;
 taper_cmd("START-TAPER worker1 $datestamp");
-like(taper_reply, qr/^TAPER-OK worker1$/,
+like(taper_reply, qr/^TAPER-OK worker1 ALLOW-TAKE-SCRIBE-FROM$/,
        "got TAPER-OK") or die;
 make_holding_file(300000, "localhost", "/u01");
 taper_cmd("FILE-WRITE worker0 $handle0 \"$test_filename\" localhost /u01 0 $datestamp 262144 \"\" \"\" \"\" \"\" \"\" \"\" \"\" 1612");
