@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 2.5.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
 
 /* Bison interface for Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989-1990, 2000-2011 Free Software Foundation, Inc.
-   
+
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,46 +26,54 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_YY_UPARSE_H_INCLUDED
+# define YY_YY_UPARSE_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 0
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     LISTHOST = 258,
-     LISTDISK = 259,
-     SETHOST = 260,
-     SETDISK = 261,
-     SETDATE = 262,
-     SETTAPE = 263,
-     SETMODE = 264,
-     CD = 265,
-     CDX = 266,
-     QUIT = 267,
-     DHIST = 268,
-     LS = 269,
-     ADD = 270,
-     ADDX = 271,
-     EXTRACT = 272,
-     LIST = 273,
-     DELETE = 274,
-     DELETEX = 275,
-     PWD = 276,
-     CLEAR = 277,
-     HELP = 278,
-     LCD = 279,
-     LPWD = 280,
-     MODE = 281,
-     SMB = 282,
-     TAR = 283,
-     PATH = 284,
-     DATE = 285
-   };
+  enum yytokentype
+  {
+    LISTHOST = 258,
+    LISTDISK = 259,
+    SETHOST = 260,
+    SETDISK = 261,
+    SETDATE = 262,
+    SETTAPE = 263,
+    SETMODE = 264,
+    CD = 265,
+    CDX = 266,
+    QUIT = 267,
+    DHIST = 268,
+    LS = 269,
+    ADD = 270,
+    ADDX = 271,
+    EXTRACT = 272,
+    LIST = 273,
+    DELETE = 274,
+    DELETEX = 275,
+    PWD = 276,
+    CLEAR = 277,
+    HELP = 278,
+    LCD = 279,
+    LPWD = 280,
+    MODE = 281,
+    SMB = 282,
+    TAR = 283,
+    PATH = 284,
+    DATE = 285
+  };
 #endif
 /* Tokens.  */
 #define LISTHOST 258
@@ -97,31 +105,27 @@
 #define PATH 284
 #define DATE 285
 
-
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
-
-/* Line 2068 of yacc.c  */
-#line 43 "uparse.y"
+#line 43 "uparse.y" /* yacc.c:1909  */
 
 	int	intval;
 	double	floatval;
 	char *	strval;
 	int	subtok;
 
-
-
-/* Line 2068 of yacc.c  */
-#line 119 "uparse.h"
-} YYSTYPE;
+#line 121 "uparse.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+
 extern YYSTYPE yylval;
 
+int yyparse (void);
 
+#endif /* !YY_YY_UPARSE_H_INCLUDED  */
