@@ -1,4 +1,4 @@
-# Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
+# Copyright (c) 2009-2013 Zmanda, Inc.  All Rights Reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -268,6 +268,7 @@ sub _make_res {
     }
 
     my $combined_res = Amanda::Changer::rait::Reservation->new(
+	$self,
 	$kid_reservations, $rait_device);
     $rait_device->read_label();
 
@@ -676,14 +677,18 @@ sub errmap (&@) {
 
 sub new {
     my $class = shift;
-    my ($child_reservations, $rait_device) = @_;
+    my ($chg, $child_reservations, $rait_device) = @_;
     my $self = Amanda::Changer::Reservation::new($class);
 
+    $self->{'chg'} = $chg;
+
     # note that $child_reservations may contain "ERROR" in place of a reservation
 
     $self->{'child_reservations'} = $child_reservations;
 
     $self->{'device'} = $rait_device;
+    my @barcodes = errmap { defined($_->{'barcode'}) ? "" . $_->{'barcode'} : "" } @$child_reservations;
+    $self->{'barcode'} = collapse_braced_alternates(\@barcodes);
 
     my @slot_names;
     @slot_names = errmap { "" . $_->{'this_slot'} } @$child_reservations;
