@@ -146,6 +146,7 @@ sub new {
 	load_poll => [0, 2, 120], # delay, poll, until
 	eject_delay => 0, # in seconds
 	unload_delay => 0, # in seconds
+	broken_drive_loaded_slot => 0,
 	class_name => $class_name,
     };
     bless ($self, $class);
@@ -236,6 +237,15 @@ sub new {
 	$self->{'drive_choice'} = $pval;
     }
 
+    # broken-drive-loaded-slot
+    my $broken_drive_loaded_slot = $self->{'config'}->get_boolean_property(
+					    "broken-drive-loaded-slot", 0);
+    if (!defined $broken_drive_loaded_slot) {
+	return Amanda::Changer->make_error("fatal", undef,
+	    message => "invalid 'broken-drive-loaded-slot' value");
+    }
+    $self->{'broken_drive_loaded_slot'} = $broken_drive_loaded_slot;
+
     # load-poll
     {
 	next unless exists $config->{'properties'}->{'load-poll'};
@@ -309,6 +319,25 @@ sub new {
     return $self;
 }
 
+sub slot_can_have_label {
+    my $self = shift;
+    my $slot = shift;
+    my $label = shift;
+
+    return 1 if !$self->{'state'};
+    my $search_barcode;
+    while (my($barcode, $blabel) = each %{$self->{'state'}->{'bc2lb'}}) {
+	$search_barcode = $barcode if $blabel eq $label;
+    }
+    my $slot_barcode = $self->{'state'}->{'slots'}->{$slot}->{'barcode'};
+    return 1 if !$search_barcode;
+    return 1 if !$slot_barcode;
+    return 0 if $search_barcode ne $slot_barcode;
+    return 1 if $search_barcode eq $slot_barcode;
+
+    return 0;
+}
+
 sub load {
     my $self = shift;
     my %params = @_;
@@ -319,6 +348,27 @@ sub load {
     $self->_with_updated_state(\%params, 'res_cb', sub { $self->load_unlocked(@_) });
 }
 
+sub set_error_to_unknown {
+    my $self  = shift;
+    my %params = @_;
+    my $state;
+    $self->with_locked_state($self->{'state_filename'},
+			     $params{'set_to_unknown_cb'}, sub {
+	my ($state, $set_to_unknown_cb) = @_;
+	for my $slot (keys %{ $state->{'slots'} }) {
+	    if (defined $state->{slots}->{$slot}->{'device_error'} and
+		$state->{slots}->{$slot}->{'device_error'} eq
+		$params{'error_message'}) {
+		$state->{slots}->{$slot}->{'device_status'} = undef;
+		$state->{slots}->{$slot}->{'device_error'} = undef;
+		$state->{slots}->{$slot}->{'label'} = undef;
+		$state->{slots}->{$slot}->{'f_type'} = undef;
+	    }
+	}
+	$set_to_unknown_cb->();
+    });
+}
+
 sub load_unlocked {
     my $self = shift;
     my %params = @_;
@@ -504,6 +554,7 @@ sub load_unlocked {
 			# not 'volinuse' because we can't expect the tape to be magically
 			# unloaded any time soon -- it's not actually in use, just inaccessible
 			reason => "invalid",
+			slot => $slot,
 			message => "the requested volume is in drive $drive, which this " .
 				   "changer instance cannot access");
 	    }
@@ -718,9 +769,7 @@ sub load_unlocked {
 			       "for '$params{label}'");
 	}
 
-	if (!$label and $params{'label'}) {
-	    $self->_debug("Expected label '$params{label}', but got an unlabeled tape");
-
+	if (!$label) {
 	    # update metadata with this new information
 	    $state->{'slots'}->{$slot}->{'state'} = Amanda::Changer::SLOT_FULL;
 	    $state->{'slots'}->{$slot}->{'device_status'} = $device->status;
@@ -739,9 +788,12 @@ sub load_unlocked {
 		delete $state->{'bc2lb'}->{$state->{'slots'}->{$slot}->{'barcode'}};
 	    }
 
-	    return $self->make_error("failed", $params{'res_cb'},
+	    if (defined $params{'label'}) {
+		$self->_debug("Expected label '$params{label}', but got an unlabeled tape");
+		return $self->make_error("failed", $params{'res_cb'},
 		    reason => "notfound",
 		    message => "Found unlabeled tape while looking for '$params{label}'");
+	    }
 	}
 
 	if (defined $self->{'tapelist'}) {
@@ -1234,10 +1286,19 @@ sub update_unlocked {
 	    # parse the string just like use-slots, using a hash for uniqueness
 	    my %changed;
 	    for my $range (split ',', $params{'changed'}) {
-		my ($first, $last) = ($range =~ /(\d+)(?:-(\d+))?/);
-		$last = $first unless defined($last);
-		for ($first .. $last) {
-		    $changed{$_} = undef;
+		if ($range eq 'error') {
+		    while (my ($sl, $inf) = each %{$state->{'slots'}}) {
+			if (defined $inf->{'device_status'} &&
+			    $inf->{'device_status'} != $DEVICE_STATUS_SUCCESS) {
+			    $changed{$sl} = undef;
+			}
+		    }
+		} else {
+		   my ($first, $last) = ($range =~ /(\d+)(?:-(\d+))?/);
+		   $last = $first unless defined($last);
+		   for ($first .. $last) {
+			$changed{$_} = undef;
+		    }
 		}
 	    }
 
@@ -1261,17 +1322,15 @@ sub update_unlocked {
 
 	my $slot = shift @slots_to_check;
 	$user_msg_fn->("Removing entry for slot $slot");
-	if (!defined $state->{'slots'}->{$slot}->{'barcode'}) {
-	    $state->{'slots'}->{$slot}->{'label'} = undef;
-	    $state->{'slots'}->{$slot}->{'device_status'} = undef;
-	    $state->{'slots'}->{$slot}->{'device_error'} = undef;
-	    $state->{'slots'}->{$slot}->{'f_type'} = undef;
-	    if (defined $state->{'slots'}->{$slot}->{'loaded_in'}) {
-		my $drive = $state->{'slots'}->{$slot}->{'loaded_in'};
-		$state->{'drives'}->{$drive}->{'label'} = undef;
-		$state->{'drives'}->{$drive}->{'state'} =
+	$state->{'slots'}->{$slot}->{'label'} = undef;
+	$state->{'slots'}->{$slot}->{'device_status'} = undef;
+	$state->{'slots'}->{$slot}->{'device_error'} = undef;
+	$state->{'slots'}->{$slot}->{'f_type'} = undef;
+	if (defined $state->{'slots'}->{$slot}->{'loaded_in'}) {
+	    my $drive = $state->{'slots'}->{$slot}->{'loaded_in'};
+	    $state->{'drives'}->{$drive}->{'label'} = undef;
+	    $state->{'drives'}->{$drive}->{'state'} =
 					Amanda::Changer::SLOT_FULL;
-	    }
 	}
 	$steps->{'set_to_unknown'}->();
     };
@@ -1337,9 +1396,11 @@ sub inventory_unlocked {
     my @inv;
     for my $slot_name (@slot_names) {
 	my $i = {};
-	next unless $self->_is_slot_allowed($slot_name);
+	my $allowed = $self->_is_slot_allowed($slot_name);
+	next if !$allowed && !$params{'all_slots'};
 	my $slot = $state->{'slots'}->{$slot_name};
 
+	$i->{'allowed'} = $allowed if $params{'all_slots'};
 	$i->{'slot'} = $slot_name;
 	$i->{'state'} = $slot->{'state'};
 	$i->{'device_status'} = $slot->{'device_status'};
@@ -1520,9 +1581,10 @@ sub verify_unlocked {
     step verify_all_unloaded => sub {
 	my ($newstate) = @_;
 
-	if (UNIVERSAL::isa($state, 'Amanda::Changer::Error')) {
+	if (UNIVERSAL::isa($newstate, 'Amanda::Changer::Error')) {
 	    return $params{'finished_cb'}->($newstate);
 	}
+	$state = $newstate;
 
 	my @loaded_drive;
 
@@ -1646,7 +1708,7 @@ sub verify_unlocked {
 	if ($device->status & $DEVICE_STATUS_VOLUME_MISSING) {
 	    debug("ERROR: Drive $drive is not device $device_name");
 	    push @results, "ERROR: Drive $drive is not device $device_name";
-	} elsif ($device->status & $DEVICE_STATUS_DEVICE_ERROR) {
+	} elsif ($device->status == $DEVICE_STATUS_DEVICE_ERROR) {
 	    debug("ERROR: Drive $drive: " . $device->error());
 	    push @results, "ERROR: Drive $drive: " . $device->error();
 	} else {
@@ -1664,10 +1726,32 @@ sub verify_unlocked {
     };
 
     step unload => sub {
-	$self->{'interface'}->unload($drive, $slot, $steps->{'unload_done'});
+	$state->{'last_status'} = 0;
+	$state->{'last_operation_time'} = 0;
+	$params{'finished_get_state_cb'} = $steps->{'unload_new_state'};
+	return $self->_get_state(\%params);
+    };
+
+    step unload_new_state => sub {
+	my ($newstate) = @_;
+
+	if (UNIVERSAL::isa($newstate, 'Amanda::Changer::Error')) {
+	    return $params{'finished_cb'}->($newstate);
+	}
+	$state = $newstate;
+
+	return $self->eject_unlocked(drive => $drive,
+			      state => $state,
+			      finished_cb => $steps->{'unload_done'});
     };
 
     step unload_done => sub {
+	my ($err) = @_;
+
+	if ($err) {
+	    debug("ERROR: $err");
+	    push @results, "ERROR: $err";
+	}
 	return $steps->{'verify_a_drive'}->();
     };
 
@@ -1684,7 +1768,11 @@ sub verify_unlocked {
 	foreach my $tape_device (@tape_devices) {
 	    $tape_devices .= " \"$tape_device\"";
 	}
-	push @results, "property \"TAPE-DEVICE\"$tape_devices";
+	if (defined $tape_devices) {
+	    push @results, "property \"TAPE-DEVICE\"$tape_devices";
+	} else {
+	    push @results, "ERROR: Found no valid tape device";
+	}
 	$params{'finished_cb'}->(undef, @results);
     };
 }
@@ -1818,6 +1906,8 @@ sub _with_updated_state {
 	}
 
 	$params{'state'} = $state;
+	# keep a copy of the latest state
+	$self->{'state'} = $state;
 	# finally, call through to the user's method; $params{$cbname} has been
 	# properly patched to release the state lock when this method is done.
 	$sub->(%params);
@@ -1924,8 +2014,11 @@ sub _get_state {
 		    debug("MISMATCH label for barcode  state ($label)   tapelist ($tl_label) for barcode $info->{'barcode'}");
 		}
 		if (!defined $label && defined $tl_label) {
-		    $label = $tl_label;
-		    $state->{'bc2lb'}->{$info->{'barcode'}} = $tl_label;
+		    if (!defined $info->{'state'} ||
+			$info->{'state'} == Amanda::Changer::SLOT_UNKNOWN) {
+			$label = $tl_label;
+			$state->{'bc2lb'}->{$info->{'barcode'}} = $tl_label;
+		    }
 		}
 		$new_slots->{$slot} = {
                     state => Amanda::Changer::SLOT_FULL,
@@ -1985,41 +2078,77 @@ sub _get_state {
 		next;
 	    }
 
-	    # trust our own orig_slot over that from the changer, if possible,
-	    # as some changers do not report this information accurately
+	if (defined $old_drive->{'orig_slot'} and
+	     defined $info->{'orig_slot'} and
+	    $old_drive->{'orig_slot'} ne $info->{'orig_slot'}) {
+		debug("WARNING: orig_slot from state file ($old_drive->{'orig_slot'}) differ from mtx output ($info->{'orig_slot'})");
+	    }
+
 	    my ($orig_slot, $label);
-	    if (defined $old_drive->{'orig_slot'}) {
-		$orig_slot = $old_drive->{'orig_slot'};
-                $label = $old_drive->{'label'};
-	    }
+	    if ($self->{'broken_drive_loaded_slot'}) {
+		# trust our own orig_slot over that from the changer, if possible,
+		# as some changers do not report this information accurately
+		if (defined $old_drive->{'orig_slot'}) {
+		    $orig_slot = $old_drive->{'orig_slot'};
+                    $label = $old_drive->{'label'};
+		}
 
-	    # but don't trust it if the barcode has changed
-	    if (defined $info->{'barcode'}
-		    and defined $old_drive->{'barcode'}
-		    and $info->{'barcode'} ne $old_drive->{'barcode'}) {
-		$orig_slot = undef;
-                $label = undef;
-	    }
+		# but don't trust it if the barcode has changed
+		if (defined $info->{'barcode'}
+		        and defined $old_drive->{'barcode'}
+		        and $info->{'barcode'} ne $old_drive->{'barcode'}) {
+		    $orig_slot = undef;
+                    $label = undef;
+		}
 
-	    # get the robot's notion of the original slot if we don't know ourselves
-	    if (!defined $orig_slot) {
+		# get the robot's notion of the original slot if we don't know ourselves
+		if (!defined $orig_slot) {
+		    $orig_slot = $info->{'orig_slot'};
+		}
+
+		# use robot slot if our slot is not empty and the changer slot is empty
+		if (defined $orig_slot and defined $info->{'orig_slot'} and $orig_slot ne $info->{'orig_slot'}
+		    and $state->{'slots'}->{$orig_slot}->{'state'} != Amanda::Changer::SLOT_EMPTY
+		    and $state->{'slots'}->{$info->{'orig_slot'}}->{'state'} == Amanda::Changer::SLOT_EMPTY) {
+		    $orig_slot = $info->{'orig_slot'};
+		}
+	    } else {
+		# use robot's slot
 		$orig_slot = $info->{'orig_slot'};
+                if (defined $orig_slot and
+                    defined $old_drive->{'orig_slot'} and
+                    $orig_slot eq $old_drive->{'orig_slot'}) {
+                    $label = $old_drive->{'label'};
+                }
 	    }
 
 	    # but if there's a tape in that slot, then we've got a problem
 	    if (defined $orig_slot
 		    and $state->{'slots'}->{$orig_slot}->{'state'} != Amanda::Changer::SLOT_EMPTY) {
-		warning("mtx indicates tape in drive $drv should go to slot $orig_slot, " .
+		warning("WARNING: mtx indicates tape in drive $drv should go to slot $orig_slot, " .
 		        "but that slot is not empty.");
 		$orig_slot = undef;
-		for my $slot (keys %{ $state->{'slots'} }) {
-		    if ($state->{'slots'}->{$slot}->{'state'} == Amanda::Changer::SLOT_EMPTY) {
-			$orig_slot = $slot;
-			last;
+		# assign an allowed slot
+		if (defined $info->{'orig_slot'} and $self->_is_slot_allowed($info->{'orig_slot'})) {
+		    for my $slot (keys %{ $state->{'slots'} }) {
+		        if ($state->{'slots'}->{$slot}->{'state'} == Amanda::Changer::SLOT_EMPTY and
+			    $self->_is_slot_allowed($slot)) {
+			    $orig_slot = $slot;
+			    last;
+			}
+		    }
+		}
+		# assign any empty slot
+		if (!defined $orig_slot) {
+		    for my $slot (keys %{ $state->{'slots'} }) {
+		        if ($state->{'slots'}->{$slot}->{'state'} == Amanda::Changer::SLOT_EMPTY) {
+			    $orig_slot = $slot;
+			    last;
+			}
 		    }
 		}
 		if (!defined $orig_slot) {
-		    warning("cannot find an empty slot for the tape in drive $drv");
+		    warning("WARNING: cannot find an empty slot for the tape in drive $drv");
 		}
 	    }
 
@@ -2027,7 +2156,8 @@ sub _get_state {
 	    if (!defined $label && defined $info->{'barcode'}) {
 		$label = $state->{'bc2lb'}->{$info->{'barcode'}};
 		my $tl_label;
-		if (defined $self->{'tapelist'}) {
+		if (defined $self->{'tapelist'} and
+		    $state->{'slots'}->{$orig_slot}->{'state'} == Amanda::Changer::SLOT_UNKNOWN) {
 		    my $tle = $self->{'tapelist'}->lookup_by_barcode($info->{'barcode'});
 		    if (defined $tle) {
 			$tl_label = $tle->{'label'};
@@ -2035,7 +2165,7 @@ sub _get_state {
 		}
 
 		if (defined $label and defined $tl_label) {
-		    debug("MISMATCH drive label for barcode  state ($label)   tapelist ($tl_label) for barcode $info->{'barcode'}");
+		    debug("WARNING: MISMATCH drive label for barcode  state ($label)   tapelist ($tl_label) for barcode $info->{'barcode'}");
 		}
 		if (!defined $label && defined $tl_label) {
 		    $label = $tl_label;
