@@ -316,6 +316,10 @@ restrict to parts with exactly this level
 
 restrict to parts with this status
 
+=item labelstr
+
+restrict to parts on volume matching the labelstr.
+
 =item dumpspecs
 
 (arrayref of dumpspecs) restruct to parts matching one or more of these dumpspecs
@@ -405,7 +409,7 @@ TODO: add_dump
 use Amanda::Logfile qw( :constants );
 use Amanda::Tapelist;
 use Amanda::Config qw( :init :getconf config_dir_relative );
-use Amanda::Util qw( quote_string weaken_ref match_disk match_host match_datestamp match_level);
+use Amanda::Util qw( quote_string weaken_ref match_disk match_host match_datestamp match_level match_labelstr_expr);
 use File::Glob qw( :glob );
 use warnings;
 use strict;
@@ -470,7 +474,7 @@ sub get_run_type {
     my ($write_timestamp) = @_;
 
     # find all of the logfiles with that name
-    my $logdir = getconf($CNF_LOGDIR);
+    my $logdir = config_dir_relative(getconf($CNF_LOGDIR));
     my @matches = File::Glob::bsd_glob("$logdir/log.$write_timestamp.*", GLOB_NOSORT);
     if ($write_timestamp =~ /000000$/) {
 	my $write_datestamp = substr($write_timestamp, 0, 8);
@@ -527,6 +531,13 @@ sub get_parts_and_dumps {
 	}
 	$params{'holding'} = 0;
     }
+    # specifying labelstr implies we won't check holding files
+    if ($params{'labelstr'}) {
+	if (defined $params{'holding'} and $params{'holding'}) {
+	    return [], []; # well, that's easy..
+	}
+	$params{'holding'} = 0;
+    }
 
     # Since we're working from logfiles, we have to pick the logfiles we'll use first.
     # Then we can use search_logfile.
@@ -620,6 +631,8 @@ sub get_parts_and_dumps {
 		and !exists($levels_hash{$find_result->{'level'}}));
 	    next if (%labels_hash 
 		and !exists($labels_hash{$find_result->{'label'}}));
+	    next if (defined $params{'labelstr'}
+		and !match_labelstr_expr($params{'labelstr'},$find_result->{'label'}));
 	    if ($get_what eq 'parts') {
 		next if (exists($params{'status'}) 
 		    and defined $find_result->{'status'}
@@ -929,7 +942,7 @@ sub add_part {
     my $logfh;
     my $logfile;
     my $find_result;
-    my $logdir = getconf($CNF_LOGDIR);
+    my $logdir = config_dir_relative(getconf($CNF_LOGDIR));
     my ($last_filenum, $last_secs, $last_kbs);
 
     # first order of business is to find out whether we need to make a new
