@@ -85,8 +85,9 @@ sub divzero_col
 sub swrite
 {
     my ( $format, @args ) = @_;
+    my @copy_args = @args;
     local $^A = "";
-    formline( $format, @args );
+    formline( $format, @copy_args );
     return $^A;
 }
 
@@ -293,7 +294,8 @@ sub calculate_stats
     if ($report->get_flag("got_finish")) {
         $total_stats->{total_time} =
              $report->get_program_info("driver",  "time", 0)
-          || $report->get_program_info("amflush", "time", 0);
+          || $report->get_program_info("amflush", "time", 0)
+          || $report->get_program_info("amvault", "time", 0);
     } else {
         $total_stats->{total_time} =
           $total_stats->{taper_time} + $total_stats->{planner_time};
@@ -382,7 +384,7 @@ sub print_header
       unless ($report->{flags}{got_finish});
 
     my $header_format = <<EOF;
-@<<<<<<<: @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<...
+@<<<<<<<: ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<...
 EOF
 
     if ($hostname) {
@@ -1196,8 +1198,14 @@ sub get_summary_info
          $dle_info->{'planner'}->{'status'} eq 'fail') or
 	($dle_info->{'driver'} &&
          $dle_info->{'driver'}->{'status'} eq 'fail')) {
-	# Do not report driver error if we have a try
-	if (!exists $alldumps->{$report->{'run_timestamp'}}) {
+
+	# Do not report driver error if we have a try with dumper
+	my $tries = $alldumps->{$report->{'run_timestamp'}};
+	my $to_report = !defined $tries || !@$tries;
+	foreach my $try ( @$tries ) {
+	    $to_report = 1 if !defined $try->{'dumper'};
+	}
+	if ($to_report) {
 	    my @rv;
 	    push @rv, 'nodump-FAILED';
 	    push @rv, $hostname;
@@ -1319,7 +1327,8 @@ sub get_summary_info
 	# pre-format the compression column, with '--' replacing 100% (i.e.,
 	# no compression)
 	my $compression;
-	if (!defined $orig_size || $orig_size == $out_size) {
+	if (!defined $orig_size || $orig_size == 0 ||
+	    ($out_size/$orig_size > 0.99  && $out_size/$orig_size < 1.01)) {
 	    $compression = '--';
 	} else {
 	    $compression =
@@ -1654,6 +1663,8 @@ sub print_if_def
 
     @$msgs or return;    # do not print section if no messages
 
+    $self->zsprint("\n");
+    $self->zsprint("\n");
     $self->zsprint("$header\n");
     foreach my $msg (@$msgs) {
         $self->zprint("  $msg\n");
