@@ -1822,7 +1822,9 @@ enum dumptypes {
 	IS_DUMP,
 	IS_GNUTAR,
 	IS_TAR,
+#ifdef SAMBA_CLIENT
 	IS_SAMBA,
+#endif
 	IS_SAMBA_TAR,
 	IS_APPLICATION_API
 };
@@ -1889,8 +1891,8 @@ extract_files_child(
     /* form the arguments to restore */
     files_off_tape = length_of_tape_list(ctl_data->elist);
     switch(dumptype) {
-    case IS_SAMBA:
 #ifdef SAMBA_CLIENT
+    case IS_SAMBA:
 	g_ptr_array_add(argv_ptr, stralloc("smbclient"));
 	smbpass = findpass(ctl_data->file.disk, &domain);
 	if (smbpass) {
@@ -1998,10 +2000,12 @@ extract_files_child(
     {
 	switch (dumptype) {
 	case IS_APPLICATION_API:
-    	case IS_TAR:
-    	case IS_GNUTAR:
-    	case IS_SAMBA_TAR:
-    	case IS_SAMBA:
+	case IS_TAR:
+	case IS_GNUTAR:
+	case IS_SAMBA_TAR:
+#ifdef SAMBA_CLIENT
+	case IS_SAMBA:
+#endif
 	    if (strcmp(fn->path, "/") == 0)
 		g_ptr_array_add(argv_ptr, stralloc("."));
 	    else
@@ -2023,7 +2027,7 @@ extract_files_child(
 	    g_ptr_array_add(argv_ptr, stralloc(fn->path));
 	    }
 	    break;
-  	}
+	}
     }
 #if defined(XFSDUMP)
     if (strcmp(ctl_data->file.program, XFSDUMP) == 0) {
@@ -2034,8 +2038,8 @@ extract_files_child(
     g_ptr_array_add(argv_ptr, NULL);
 
     switch (dumptype) {
-    case IS_SAMBA:
 #ifdef SAMBA_CLIENT
+    case IS_SAMBA:
     	cmd = stralloc(SAMBA_CLIENT);
     	break;
 #else
@@ -2684,6 +2688,7 @@ read_amidxtaped_data(
     void *	buf,
     ssize_t	size)
 {
+    size_t count;
     ctl_data_t *ctl_data = (ctl_data_t *)cookie;
     assert(cookie != NULL);
 
@@ -2788,7 +2793,12 @@ read_amidxtaped_data(
 	/*
 	 * We ignore errors while writing to the index file.
 	 */
-	(void)full_write(ctl_data->child_pipe[1], buf, (size_t)size);
+	count = full_write(ctl_data->child_pipe[1], buf, (size_t)size);
+	if (count != (size_t)size) {
+	    g_debug("Failed to write to application: %s", strerror(errno));
+	    g_printf("Failed to write to application: %s\n", strerror(errno));
+	    stop_amidxtaped();
+	}
     }
 }
 
