@@ -90,7 +90,9 @@ sub _user_msg_fn {
             print STDERR "Searching for label '$params{'label'}':";
         } elsif (exists($params{'slot_result'}) ||
                  exists($params{'search_result'})) {
-            if (defined($params{'err'}) and ref($params{'err'}) eq "HASH") {
+            if (defined($params{'err'}) and
+		(ref($params{'err'}) eq "HASH" ||
+		 ref($params{'err'}) eq "Amanda::Changer::Error")) {
                 if (exists($params{'search_result'}) &&
                     defined($params{'err'}->{'this_slot'})) {
                     print STDERR "slot $params{'err'}->{'this_slot'}:";
@@ -262,7 +264,7 @@ sub do_check {
 	    return $steps->{'release'}->();
 	}
 
-	print "Writing label '$label' to check writablility\n";
+	print "Writing label '$label' to check writability\n";
 	if (!$res->{'device'}->start($ACCESS_WRITE, $label, "X")) {
 	    print "ERROR: writing to volume: " . $res->{'device'}->error_or_status(), "\n";
 	    $exit_status = 1;
