@@ -59,6 +59,8 @@ static int overwrite;
 static disklist_t origq;
 
 static uid_t uid_dumpuser;
+static gboolean who_check_host_setting = TRUE;//  TRUE => local check auth
+                                      // FALSE => clent check auth
 
 /* local functions */
 
@@ -68,6 +70,7 @@ pid_t start_server_check(int fd, int do_localchk, int do_tapechk);
 int main(int argc, char **argv);
 int check_tapefile(FILE *outf, char *tapefile);
 int test_server_pgm(FILE *outf, char *dir, char *pgm, int suid, uid_t dumpuid);
+static int check_host_setting(FILE *outf);
 
 void
 usage(void)
@@ -134,6 +137,7 @@ main(
 
     safe_fd(-1, 0);
     safe_cd();
+    make_crc_table();
 
     set_pname("amcheck");
     /* drop root privileges */
@@ -307,9 +311,9 @@ main(
     }
 #ifdef CHECK_USERID
     if (uid_me != uid_dumpuser) {
-	error(_("running as user \"%s\" instead of \"%s\".\n"
-		"Change user to \"%s\" or change dump user to \"%s\" in amanda.conf"),
-	      pw->pw_name, dumpuser, dumpuser, pw->pw_name);
+	error("must be executed as the \"%s\" user instead of the \"%s\" user.\n"
+		"Change user to \"%s\" or change dumpuser to \"%s\" in amanda.conf",
+	      dumpuser, pw->pw_name, dumpuser, pw->pw_name);
         /*NOTREACHED*/
     }
 #endif
@@ -350,6 +354,8 @@ main(
 	/* just use stdout */
 	mainfd = 1;
 
+    who_check_host_setting = do_localchk;
+
     /* start server side checks */
 
     if(do_localchk || do_tapechk)
@@ -453,8 +459,8 @@ main(
 	}
 	if(mailto) {
 	    a = (char **) g_new0(char *, 2);
-	    a[1] = stralloc(mailto);
-	    a[2] = NULL;
+	    a[0] = stralloc(mailto);
+	    a[1] = NULL;
 	} else {
 	    /* (note that validate_mailto doesn't allow any quotes, so this
 	     * is really just splitting regular old strings) */
@@ -761,6 +767,10 @@ start_server_check(
         tp = lookup_tapetype(getconf_str(CNF_TAPETYPE));
     }
 
+    if (who_check_host_setting) {
+	check_host_setting(outf);
+    }
+
     /*
      * Check various server side config file settings.
      */
@@ -2150,11 +2160,15 @@ start_client_checks(
     g_fprintf(outf, _("\nAmanda Backup Client Hosts Check\n"));
     g_fprintf(outf,   "--------------------------------\n");
 
+    hostcount = remote_errors = 0;
+
+    if (!who_check_host_setting) {
+	remote_errors = check_host_setting(outf);
+    }
+
     run_server_global_scripts(EXECUTE_ON_PRE_AMCHECK, get_config_name());
     protocol_init();
 
-    hostcount = remote_errors = 0;
-
     for(dp = origq.head; dp != NULL; dp = dp->next) {
 	hostp = dp->host;
 	if(hostp->up == HOST_READY && dp->todo == 1) {
@@ -2316,3 +2330,35 @@ handle_result(
        them explicitly */
     while(waitpid(-1, NULL, WNOHANG)> 0);
 }
+
+static int
+check_host_setting(
+    FILE *outf)
+{
+    am_host_t *p;
+    disk_t *dp;
+    int count = 0;
+
+    for (p = get_hostlist(); p != NULL; p = p->next) {
+	for(dp = p->disks; dp != NULL; dp = dp->hostnext) {
+	    if (strcmp(dp->auth, p->disks->auth) != 0) {
+		g_fprintf(outf, "ERROR: Multiple DLE's for host '%s' use different auth methods\n",
+			  p->hostname);
+		g_fprintf(outf, "       Please ensure that all DLE's for the host use the same auth method, including skipped ones\n");
+		count++;
+		break;
+	    }
+	}
+
+	for(dp = p->disks; dp != NULL; dp = dp->hostnext) {
+	    if (dp->maxdumps != p->disks->maxdumps) {
+		g_fprintf(outf, "ERROR: Multiple DLE's for host '%s' use different maxdumps values\n",
+			  p->hostname);
+		g_fprintf(outf, "       Please ensure that all DLE's for the host use the same maxdumps value, including skipped ones\n");
+		count++;
+		break;
+	    }
+	}
+    }
+    return count;
+}
