@@ -26,6 +26,7 @@ use Getopt::Long;
 use File::Basename;
 use XML::Simple;
 use IPC::Open3;
+use MIME::Base64 ();
 
 use Amanda::Device qw( :constants );
 use Amanda::Debug qw( :logging );
@@ -203,7 +204,7 @@ usage("The -b option is no longer supported; set readblocksize in the tapetype s
 usage("-l is not compatible with -c or -C")
     if ($opt_leave and $opt_compress);
 usage("-p is not compatible with -n")
-    if ($opt_leave and $opt_no_reassembly);
+    if ($opt_pipe and $opt_no_reassembly);
 usage("-h, --header-file, and --header-fd are mutually incompatible")
     if (($opt_header and ($opt_header_file or $opt_header_fd))
 	    or ($opt_header_file and $opt_header_fd));
@@ -431,6 +432,26 @@ sub main {
 	    $delay = 5000; # 5 seconds
 	}
 
+	my $_user_msg_fn = sub {
+	    my %params = @_;
+
+	    if (exists($params{'slot_result'})) {
+		if (defined($params{'err'})) {
+		    print "slot $params{'slot'}: $params{'err'}\n";
+		} else { # res must be defined
+		    my $res = $params{'res'};
+		    my $dev = $res->{'device'};
+		    if ($dev->status == $DEVICE_STATUS_SUCCESS) {
+			#my $volume_label = $res->{device}->volume_label;
+			#print "slot $params{'slot'}: $volume_label\n";
+		    } else {
+			my $errmsg = $res->{device}->error_or_status();
+			print "slot $params{'slot'}: $errmsg\n";
+		    }
+		}
+	    }
+	};
+
 	my $interactivity = Amanda::Interactivity::amfetchdump->new();
 	# if we have an explicit device, then the clerk doesn't get a changer --
 	# we operate the changer via Amanda::Recovery::Scan
@@ -439,6 +460,7 @@ sub main {
 	    return failure($chg, $finished_cb) if $chg->isa("Amanda::Changer::Error");
 	    my $scan = Amanda::Recovery::Scan->new(
 				chg => $chg,
+				user_msg_fn => $_user_msg_fn,
 				interactivity => $interactivity);
 	    return failure($scan, $finished_cb) if $scan->isa("Amanda::Changer::Error");
 	    $clerk = Amanda::Recovery::Clerk->new(
@@ -446,6 +468,7 @@ sub main {
 		scan     => $scan);
 	} else {
 	    my $scan = Amanda::Recovery::Scan->new(
+				user_msg_fn => $_user_msg_fn,
 				interactivity => $interactivity);
 	    return failure($scan, $finished_cb) if $scan->isa("Amanda::Changer::Error");
 
@@ -530,6 +553,10 @@ sub main {
 		print "ERROR: XML error\n";
 		debug("XML Error: $@\n$dle_str");
 	    }
+	    if (defined $dle->{'diskdevice'} and UNIVERSAL::isa( $dle->{'diskdevice'}, "HASH" )) {
+		$dle->{'diskdevice'} = MIME::Base64::decode($dle->{'diskdevice'}->{'raw'});
+	    }
+
 	}
 
 	# and set up the destination..
