@@ -249,7 +249,7 @@ uncompress_file(
 
 	indexfd = open(filename,O_WRONLY|O_CREAT, 0600);
 	if (indexfd == -1) {
-	    msg = vstrallocf(_("Can't open '%s' for writting: %s"),
+	    msg = vstrallocf(_("Can't open '%s' for writing: %s"),
 			      filename, strerror(errno));
 	    dbprintf("%s\n", msg);
 	    g_ptr_array_add(*emsg, msg);
@@ -1557,7 +1557,7 @@ main(
 	    user_validated = amindexd_debug ||
 				check_security(
 					(sockaddr_union *)&his_addr,
-					arg, 0, &errstr);
+					arg, 0, &errstr, "amindexd");
 	    if(user_validated) {
 		reply(200, _("Access OK"));
 		amfree(line);
