@@ -202,7 +202,7 @@ $label = "";					# -w fodder
 $origsize = 0;					# -w fodder
 $idle_dumpers = 0;
 $status_driver = "";
-$status_taper = "Searching for a new tape";
+$status_taper->{0} = "Idle";
 $estimate_done = 0;
 $holding_space = 0;
 $start_time = 0;
@@ -452,32 +452,36 @@ while($lineX = <AMDUMP>) {
 			elsif($line[5] =~ /taper/) {
 				if($line[6] eq "START-TAPER") {
 					#7:name 8:timestamp
+					$worker = $line[7];
 					$gdatestamp=$line[8];
 					if(!defined $datestamp{$gdatestamp}) {
 						$datestamp{$gdatestamp} = 1;
 						push @datestamp, $gdatestamp;
 					}
-					$status_taper = "Searching for a new tape";
+					$status_taper{$worker} = "Searching for a new tape";
 				}
 				elsif($line[6] eq "NEW-TAPE") {
 					#7:name 8:handle
-					$status_taper = "Searching for a new tape";
+					$worker = $line[7];
+					$status_taper{$worker} = "Searching for a new tape";
 				}
 				elsif($line[6] eq "NO-NEW-TAPE") {
 					#7:name 8:handle 9:errmsg
+					$worker = $line[7];
 					$serial=$line[8];
 					$error=$line[9];
-					$status_taper = $error;
+					$status_taper{$worker} = $error;
 				}
 				elsif($line[6] eq "FILE-WRITE") {
 					#7:name 8:handle 9:filename 10:host 11:disk 12:level 13:datestamp 14:splitsize
+					$worker = $line[7];
 					$name=$line[7];
 					$serial=$line[8];
 					$host=$line[10];
 					$partition=$line[11];
 					$level=$line[12];
 					$ldatestamp=$line[13];
-					$status_taper = "Writing $host:$partition";
+					$status_taper{$worker} = "Writing $host:$partition";
 					if(!defined $datestamp{$ldatestamp}) {
 						$datestamp{$ldatestamp} = 1;
 						push @datestamp, $ldatestamp;
@@ -498,12 +502,13 @@ while($lineX = <AMDUMP>) {
 				elsif($line[6] eq "PORT-WRITE") {
 					#7:name 8:handle 9:host 10:disk 11:level 12:datestamp 13:splitsize 14:diskbuffer 15:fallback_splitsize
 					$name=$line[7];
+					$worker = $line[7];
 					$serial=$line[8];
 					$host=$line[9];
 					$partition=$line[10];
 					$level=$line[11];
 					$ldatestamp=$line[12];
-					$status_taper = "Writing $host:$partition";
+					$status_taper{$worker} = "Writing $host:$partition";
 					$hostpart=&make_hostpart($host,$partition,$ldatestamp);
 					$serial{$serial}=$hostpart;
 					$taper_started{$hostpart}=1;
@@ -517,8 +522,10 @@ while($lineX = <AMDUMP>) {
 				}
 				elsif($line[6] eq "TAKE-SCRIBE-FROM") {
 					#7:name1 #8:handle #9:name2
+					$worker1 = $line[7];
 					$name1=$line[7];
 					$serial=$line[8];
+					$worker2 = $line[7];
 					$name2=$line[9];
 					$hostpart=$serial{$serial};
 					$taper_nb{$name1} = $taper_nb{$name2};
@@ -526,6 +533,8 @@ while($lineX = <AMDUMP>) {
 					if (defined $hostpart) {
 						$error{$hostpart} = $olderror{$hostpart};
 					}
+					$status_taper{$worker1} = $status_taper{$worker2};
+					$status_taper{$worker2} = "Idle";
 				}
 			}
 		}
@@ -642,7 +651,9 @@ while($lineX = <AMDUMP>) {
 						if(defined $hostpart) {
 							$error= "taper CRASH";
 							$taper_finished{$hostpart} = -2;
-							$status_taper = $error;
+							foreach $worker (keys %status_taper) {
+								$status_taper{$worker} = $error;
+							}
 							$busy_time{"taper"}+=($current_time-$taper_time{$hostpart});
 							$taper_time{$hostpart}=$current_time;
 							$error{$hostpart}="$error";
@@ -656,8 +667,9 @@ while($lineX = <AMDUMP>) {
 					#PARTIAL: 7:handle 8:INPUT-* 9:TAPE-* 10:errstr 11:INPUT-MSG 12:TAPE-MSG
 					$serial=$line[7];
 
-					$status_taper = "Idle";
 					$hostpart=$serial{$serial};
+					$worker = $taper_name{$hostpart};
+					$status_taper{$worker} = "Idle";
 					$line[10] =~ /sec (\S+) (kb|bytes) (\d+) kps/;
 					if ($2 eq 'kb') {
 						$size=$3 / $unitdivisor;
@@ -701,9 +713,10 @@ while($lineX = <AMDUMP>) {
 				elsif($line[6] eq "REQUEST-NEW-TAPE") {
 					#7:serial
 					$serial=$line[7];
-					$old_status_taper = $status_taper;
-					$status_taper = "Asking for a new tape";
 					$hostpart=$serial{$serial};
+					$worker = $taper_name{$hostpart};
+					$old_status_taper{$worker} = $status_taper{$worker};
+					$status_taper{$worker} = "Asking for a new tape";
 					if (defined $hostpart and
 						!defined($olderror{$hostpart})) {
 						$olderror{$hostpart} = $error{$hostpart};
@@ -713,8 +726,9 @@ while($lineX = <AMDUMP>) {
 				elsif($line[6] eq "NEW-TAPE") {
 					#7:serial #8:label
 					$serial=$line[7];
-					$status_taper = $old_status_taper;
 					$hostpart=$serial{$serial};
+					$worker = $taper_name{$hostpart};
+					$status_taper{$worker} = $old_status_taper{$worker};
 					$nb_tape++;
 					$taper_nb{$taper_name{$hostpart}} = $nb_tape;
 					$label = $line[8];
@@ -728,12 +742,14 @@ while($lineX = <AMDUMP>) {
 				}
 				elsif($line[6] eq "TAPER-OK") {
 					#7:name #8:label
-					$status_taper = "Idle";
+					$worker = $line[7];
+					$status_taper{$worker} = "Idle";
 				}
 				elsif($line[6] eq "TAPE-ERROR") {
 					#7:name 8:errstr
+					$worker = $line[7];
 					$error=$line[8];
-					$status_taper = $error;
+					$status_taper{$worker} = $error;
 					$exit_status |= $STATUS_TAPE;
 					undef $taper_status_file{$hostpart};
 				}
@@ -741,22 +757,23 @@ while($lineX = <AMDUMP>) {
 					#7:handle 8:INPUT- 9:TAPE- 10:input_message 11:tape_message
 					$serial=$line[7];
 					$hostpart=$serial{$serial};
+					$worker = $taper_name{$hostpart};
 					if(defined $hostpart) {
 						if($line[9] eq "TAPE-ERROR") {
 							$error=$line[11];
 							$taper_finished{$hostpart} = -2;
-							$status_taper = $error;
+							$status_taper{$worker} = $error;
 						} elsif($line[9] eq "TAPE-CONFIG") {
 							$tape_config{$hostpart} = $error;
 							$error=$line[11];
 							$tape_config{$hostpart} = $error;
 							$taper_finished{$hostpart} = -2;
-							$status_taper = $error;
+							$status_taper{$worker} = $error;
 						} else { # INPUT-ERROR
 							$error = $line[10];
 							$error = $error{$hostpart} if defined $error{$hostpart};
 							$taper_finished{$hostpart} = -1;
-							$status_taper = "Idle";
+							$status_taper{$worker} = "Idle";
 						}
 						$busy_time{"taper"}+=($current_time-$taper_time{$hostpart});
 						$taper_time{$hostpart}=$current_time;
@@ -963,7 +980,7 @@ foreach $host (sort @hosts) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s", "$host:$qpartition";
 							printf "%2d ",  $level{$hostpart};
-							printf "%9d$unit", $esize{$hostpart};
+							printf "%9s$unit", $esize{$hostpart};
 							if($partialestimate{$hostpart} == 1) {
 								if ($dead_run) {
 									print " failed: killed while";
@@ -1008,7 +1025,7 @@ foreach $host (sort @hosts) {
 						if(defined $opt_failed) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-							printf "%9d$unit", $esize{$hostpart};
+							printf "%9s$unit", $esize{$hostpart};
 							print " dump to tape failed: " . $error{$hostpart};
 							print "\n";
 						}
@@ -1023,7 +1040,7 @@ foreach $host (sort @hosts) {
 							 defined $opt_failed && $dead_run != 0) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-							printf "%9d$unit", $esize{$hostpart};
+							printf "%9s$unit", $esize{$hostpart};
 							if ($dead_run) {
 								print " failed: killed while";
 								$exit_status |= $STATUS_FAILED;
@@ -1043,7 +1060,7 @@ foreach $host (sort @hosts) {
 								close FF;
 							}
 							if(defined($size)) {
-								printf " (%d$unit done (%0.2f%%))", $size, 100.0 * $size/$esize{$hostpart};
+								printf " (%s$unit done (%0.2f%%))", $size, 100.0 * $size/$esize{$hostpart};
 								$dtsize += $size;
 							}
 							if( defined $starttime ) {
@@ -1059,7 +1076,7 @@ foreach $host (sort @hosts) {
 							 defined $opt_failed && $dead_run != 0) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-							printf "%9d$unit", $size{$hostpart};
+							printf "%9s$unit", $size{$hostpart};
 							if ($dead_run) {
 								print " failed: killed while";
 								$exit_status |= $STATUS_FAILED;
@@ -1094,7 +1111,7 @@ foreach $host (sort @hosts) {
 								close FF;
 							}
 							if(defined($size) and defined($size{$hostpart}) and $size{$hostpart} > 0) {
-								printf " (%d$unit done (%0.2f%%))", $size, 100.0 * $size/$size{$hostpart};
+								printf " (%s$unit done (%0.2f%%))", $size, 100.0 * $size/$size{$hostpart};
 							}
 							if( defined $starttime ) {
 								print " (", &showtime($taper_time{$hostpart}), ")";
@@ -1146,7 +1163,7 @@ foreach $host (sort @hosts) {
 							 (defined $opt_waittaper && ($taper_finished{$hostpart} == -1))) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-							printf "%9d$unit", $xsize;
+							printf "%9s$unit", $xsize;
 							print " dump done," if defined $dump_finished{$hostpart} && $dump_finished{$hostpart} == 1;
 							if($in_flush == 0) {
 								if ($tape_config{$hostpart}) {
@@ -1199,7 +1216,7 @@ foreach $host (sort @hosts) {
 						if( defined $opt_finished ) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-							printf "%9d$unit", $size{$hostpart};
+							printf "%9s$unit", $size{$hostpart};
 							if($in_flush == 0) {
 								if (defined $dump_finished{$hostpart} && $dump_finished{$hostpart} == -3) {
 									$exit_status |= $STATUS_FAILED;
@@ -1262,7 +1279,7 @@ foreach $host (sort @hosts) {
 							if( defined $opt_waitdumping ) {
 								printf "%8s ", $datestamp if defined $opt_date;
 								printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-								printf "%9d$unit", $esize{$hostpart};
+								printf "%9s$unit", $esize{$hostpart};
 								if ($dead_run) {
 									print " failed: process terminated while";
 									$exit_status |= $STATUS_FAILED;
@@ -1298,12 +1315,12 @@ foreach $host (sort @hosts) {
 							 defined $opt_failed && $dead_run != 0) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-							printf "%9d$unit", $esize{$hostpart};
+							printf "%9s$unit", $esize{$hostpart};
 							if ($dead_run) {
 								print " failed: killed while";
 								$exit_status |= $STATUS_FAILED;
 							}
-							printf " dumping %8d$unit", $size{$hostpart};
+							printf " dumping %8s$unit", $size{$hostpart};
 							if($size{$hostpart} != 0) {
 								printf " (%6.2f%%)", (100.0*$size{$hostpart})/$esize{$hostpart};
 							}
@@ -1324,7 +1341,7 @@ foreach $host (sort @hosts) {
 						if( defined $opt_waittaper ) {
 							printf "%8s ", $datestamp if defined $opt_date;
 							printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-							printf "%9d$unit", $size{$hostpart};
+							printf "%9s$unit", $size{$hostpart};
 							print " dump done";
 							if( defined $starttime ) {
 								print " (", &showtime($dump_time{$hostpart}), ")";
@@ -1357,7 +1374,7 @@ foreach $host (sort @hosts) {
 					if( defined $opt_waittaper ) {
 						printf "%8s ", $datestamp if defined $opt_date;
 						printf "%-${maxnamelength}s%2d ", "$host:$qpartition", $level{$hostpart};
-						printf "%9d$unit", $size{$hostpart};
+						printf "%9s$unit", $size{$hostpart};
 						if ($dead_run) {
 							print " process terminated while";
 						}
@@ -1386,59 +1403,59 @@ if (defined $opt_summary) {
 	print  "SUMMARY          part      real  estimated\n";
 	print  "                           size       size\n";
 	printf "partition       : %3d\n", $nb_partition;
-	printf "estimated       : %3d %20d$unit\n", $epartition , $estsize;
-	printf "flush           : %3d %9d$unit\n", $flpartition, $flsize;
-	printf "failed          : %3d %20d$unit           (%6.2f%%)\n",
-		$fpartition , $fsize,
+	printf "estimated       : %3d %20s$unit\n", $epartition , int($estsize);
+	printf "flush           : %3d %9s$unit\n", $flpartition, int($flsize);
+	printf "failed          : %3d %20s$unit           (%6.2f%%)\n",
+		$fpartition , int($fsize),
 		$estsize ? ($fsize * 1.0 / $estsize) * 100 : 0.0;
-	printf "wait for dumping: %3d %20d$unit           (%6.2f%%)\n",
-		$wpartition , $wsize,
+	printf "wait for dumping: %3d %20s$unit           (%6.2f%%)\n",
+		$wpartition , int($wsize),
 		$estsize ? ($wsize * 1.0 / $estsize) * 100 : 0.0;
 	if(defined($dtsize)) {
-		printf "dumping to tape : %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-			$dtpartition, $dtsize, $dtesize,
+		printf "dumping to tape : %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+			$dtpartition, int($dtsize), int($dtesize),
 			$dtsize ? ($dtsize * 1.0 / $dtesize) * 100 : 0.0,
 			$estsize ? ($dtesize * 1.0 / $estsize) * 100 : 0.0;
 	} else {
-		printf "dumping to tape : %3d %20d$unit           (%6.2f%%)\n",
-			$dtpartition, $dtesize,
+		printf "dumping to tape : %3d %20dsunit           (%6.2f%%)\n",
+			$dtpartition, int($dtesize),
 			$estsize ? ($dtesize * 1.0 / $estsize) * 100 : 0.0;
 	}
-	printf "dumping         : %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-		$dupartition, $dusize, $duesize,
+	printf "dumping         : %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+		$dupartition, int($dusize), int($duesize),
 		$duesize ? ($dusize * 1.0 / $duesize) * 100 : 0.0,
 		$estsize ? ($dusize * 1.0 / $estsize) * 100 : 0.0;
-	printf "dumped          : %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-		$dpartition , $dsize , $desize,
+	printf "dumped          : %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+		$dpartition , int($dsize) , int($desize),
 		$desize ? ($dsize * 1.0 / $desize) * 100 : 0.0,
 		$estsize ? ($dsize * 1.0 / $estsize) * 100 : 0.0;
-	printf "wait for writing: %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-		$twpartition, $twsize, $twesize,
+	printf "wait for writing: %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+		$twpartition, int($twsize), int($twesize),
 		$twesize ? ($twsize * 1.0 / $twesize) * 100 : 0.0,
 		$estsize ? ($twsize * 1.0 / $estsize) * 100 : 0.0;
-	printf "wait to flush   : %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-		$wfpartition, $wfsize, $wfsize, 100, 0;
-	printf "writing to tape : %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-		$tapartition, $tasize, $taesize,
+	printf "wait to flush   : %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+		$wfpartition, int($wfsize), int($wfsize), 100, 0;
+	printf "writing to tape : %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+		$tapartition, int($tasize), int($taesize),
 		$taesize ? ($tasize * 1.0 / $taesize) * 100 : 0.0,
 		$estsize ? ($tasize * 1.0 / $estsize) * 100 : 0.0;
-	printf "failed to tape  : %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-		$tfpartition, $tfsize, $tfesize,
+	printf "failed to tape  : %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+		$tfpartition, int($tfsize), int($tfesize),
 		$tfesize ? ($tfsize * 1.0 / $tfesize) * 100 : 0.0,
 		$estsize ? ($tfsize * 1.0 / $estsize) * 100 : 0.0;
-	printf "taped           : %3d %9d$unit %9d$unit (%6.2f%%) (%6.2f%%)\n",
-		$tpartition , $tsize , $tesize,
+	printf "taped           : %3d %9s$unit %9s$unit (%6.2f%%) (%6.2f%%)\n",
+		$tpartition , int($tsize) , int($tesize),
 		$tesize ? ($tsize * 1.0 / $tesize) * 100 : 0.0,
 		($estsize+$flsize) ? ($tsize * 1.0 / ($estsize + $flsize)) * 100 : 0.0;
 	if($nb_tape > 1 || $tape_size != 0) {
 		for($i=1; $i <= $nb_tape; $i++) {
 			if($tape_size != 0) {
-				printf "  tape %-3d      : %3d %9d$unit %9d$unit (%6.2f%%) %s",
-					$i, $ntpartition{$i}, $ntsize{$i}, $ntesize{$i}, 100*$ntsize{$i}/$tape_size, $ntlabel{$i};
+				printf "  tape %-3d      : %3d %9s$unit %9s$unit (%6.2f%%) %s",
+					$i, $ntpartition{$i}, int($ntsize{$i}), int($ntesize{$i}), 100*$ntsize{$i}/$tape_size, $ntlabel{$i};
 			}
 			else {
-				printf "  tape %-3d      : %3d %9d$unit %9d$unit %s",
-					$i, $ntpartition{$i}, $ntsize{$i}, $ntesize{$i}, $ntlabel{$i};
+				printf "  tape %-3d      : %3d %9s$unit %9s$unit %s",
+					$i, $ntpartition{$i}, int($ntsize{$i}), int($ntesize{$i}), $ntlabel{$i};
 			}
 			if(defined($ntchunk{$i}) && $ntchunk{$i} > 0) {
 				printf " (%d chunks)", $ntchunk{$i};
@@ -1456,7 +1473,12 @@ if (defined $opt_summary) {
 		printf "%d dumper%s idle%s %s: %s\n", $idle_dumpers, $c1, $c2, $c3, $status_driver;
 	}
 
-	printf "taper status: $status_taper\n";
+	foreach $worker (sort keys %status_taper) {
+		$num = $worker;
+	   $num =~ s/worker//g;
+		print "taper $num status: $status_taper{$worker}\n";
+   }
+	#printf "taper status: $status_taper\n";
 	if (defined $qlen{"tapeq"}) {
 		printf "taper qlen: %d\n", $qlen{"tapeq"};
 	}
@@ -1469,12 +1491,13 @@ if (defined $opt_summary) {
 		} else {
 			$hs = 0.0;
 		}
-		printf "holding space   : %9d$unit (%6.2f%%)\n", ($free{"space"}/$unitdivisor), $hs;
+		printf "holding space   : %9s$unit (%6.2f%%)\n", int($free{"space"}/$unitdivisor), $hs;
 	}
 }
 
 if(defined $opt_stats) {
-	if(defined($current_time) && $current_time != $start_time) {
+	if(defined($current_time) and defined($start_time) and
+	   $current_time != $start_time) {
 		$total_time=$current_time-$start_time;
 		foreach $key (sort byprocess keys %busy_time) {
 			printf "%8s busy   : %8s  (%6.2f%%)\n",
