@@ -34,6 +34,11 @@ sub new {
     if (!-r STDIN) {
 	return undef;
     }
+    my $stdin;
+    if (!open $stdin, "<&STDIN") {
+     return undef;
+    }
+    close $stdin;
 
     my $self = {
 	input_src => undef};
@@ -100,6 +105,7 @@ package Amvault;
 use Amanda::Config qw( :getconf config_dir_relative );
 use Amanda::Disklist;
 use Amanda::Debug qw( :logging debug );
+use Amanda::Device qw( :constants );;
 use Amanda::Xfer qw( :constants );
 use Amanda::Header qw( :constants );
 use Amanda::MainLoop;
@@ -122,6 +128,8 @@ use base qw(
     Amanda::Taper::Scribe::Feedback
 );
 
+eval "require Time::HiRes qw / time /;";
+
 sub new {
     my $class = shift;
     my %params = @_;
@@ -132,11 +140,13 @@ sub new {
 	latest_fulls => $params{'latest_fulls'},
 	incrs_only => $params{'incrs_only'},
 	opt_export => $params{'opt_export'},
+	opt_interactivity => $params{'opt_interactivity'},
 	opt_dumpspecs => $params{'opt_dumpspecs'},
 	opt_dry_run => $params{'opt_dry_run'},
 	config_name => $params{'config_name'},
 
 	src_write_timestamp => $params{'src_write_timestamp'},
+	src_labelstr => $params{'src_labelstr'},
 
 	dst_changer => $params{'dst_changer'},
 	dst_autolabel => $params{'dst_autolabel'},
@@ -235,6 +245,7 @@ sub run {
 	    return $self->bail_already_running();
 	}
 	close($tl);
+	$self->{'start_time'} = time;
 	log_add($L_START, "date " . $self->{'dst_write_timestamp'});
 	Amanda::Debug::add_amanda_log_handler($amanda_log_trace_log);
 	$self->{'cleanup'}{'roll_trace_log'} = 1;
@@ -257,10 +268,35 @@ sub setup_src {
 
     $src->{'seen_labels'} = {};
 
-    $src->{'interactivity'} = main::Interactivity->new();
+    if ($self->{'opt_interactivity'}) {
+	$src->{'interactivity'} = main::Interactivity->new();
+    }
+
+    my $_user_msg_fn = sub {
+	my %params = @_;
+            
+	if (exists($params{'slot_result'})) {
+	    if (defined($params{'err'})) {
+		print "slot $params{'slot'}: $params{'err'}\n";
+		log_add($L_INFO, "slot $params{'slot'}: $params{'err'}");
+	    } else { # res must be defined
+		my $res = $params{'res'};
+		my $dev = $res->{'device'};
+		if ($dev->status == $DEVICE_STATUS_SUCCESS) {
+		    #my $volume_label = $res->{device}->volume_label;
+		    #print "slot $params{'slot'}: $volume_label\n";
+		} else {
+		    my $errmsg = $res->{device}->error_or_status();
+		    print "slot $params{'slot'}: $errmsg\n";
+		    log_add($L_INFO, "slot $params{'slot'}: $errmsg");
+		}
+	    }
+	}
+    };
 
     $src->{'scan'} = Amanda::Recovery::Scan->new(
 	    chg => $src->{'chg'},
+	    user_msg_fn => $_user_msg_fn,
 	    interactivity => $src->{'interactivity'});
 
     $src->{'clerk'} = Amanda::Recovery::Clerk->new(
@@ -370,6 +406,7 @@ sub setup_src {
     Amanda::Recovery::Planner::make_plan(
 	    latest_fulls => $self->{'latest_fulls'},
 	    dumpspecs => \@dumpspecs,
+	    src_labelstr => $self->{'src_labelstr'},
 	    changer => $src->{'chg'},
 	    plan_cb => sub { $self->plan_cb(@_) });
 }
@@ -383,14 +420,6 @@ sub plan_cb {
 
     $src->{'plan'} = $plan;
 
-    @{$plan->{'dumps'}} = grep {
-	    Amanda::Disklist::get_disk($_->{'hostname'}, $_->{'diskname'})
-	} @{$plan->{'dumps'}};
-    if (!@{$plan->{'dumps'}}) {
-	return $self->failure("No dumps to vault");
-    }
-
-
     if ($self->{'opt_dry_run'}) {
 	my $total_kb = Math::BigInt->new(0);
 
@@ -535,8 +564,20 @@ sub xfer_dumps {
         my ($errors, $header, $xfer_src_, $directtcp_supported) = @_;
 	$xfer_src = $xfer_src_;
 
-	return $finished_cb->(join("\n", @$errors))
-	    if $errors;
+	if ($errors) {
+	    my $msg = join("; ", @$errors);
+	    my $dump = $current->{'dump'};
+	    log_add_full($L_FAIL, "taper", sprintf("%s %s %s %s %s %s",
+		quote_string($dump->{'hostname'}.""), # " is required for SWIG..
+		quote_string($dump->{'diskname'}.""),
+		$dump->{'dump_timestamp'},
+		$dump->{'level'},
+                'error',
+                $msg));
+	    $self->vlog($msg);
+	    # next dump
+	    return $steps->{'get_dump'}->();
+	}
 
 	$current->{'header'} = $header;
 
@@ -587,19 +628,21 @@ sub xfer_dumps {
 	$size = $current->{'dump'}->{'bytes'} if exists $current->{'dump'}->{'bytes'};
 	$xfer->start($steps->{'handle_xmsg'}, 0, $size);
 
-	$self->{'timer'} = Amanda::MainLoop::timeout_source($self->{'delay'});
-	$self->{'timer'}->set_callback(sub {
-	    my $size = $dst->{'scribe'}->get_bytes_written();
-	    if ($self->{'is_tty'}) {
-		if (!$self->{'last_is_size'}) {
-		    print STDERR "\n";
-		    $self->{'last_is_size'} = 1;
+	if (!$self->{'timer'}) {
+	    $self->{'timer'} = Amanda::MainLoop::timeout_source($self->{'delay'});
+	    $self->{'timer'}->set_callback(sub {
+		my $size = $dst->{'scribe'}->get_bytes_written();
+		if ($self->{'is_tty'}) {
+		    if (!$self->{'last_is_size'}) {
+			print STDERR "\n";
+			$self->{'last_is_size'} = 1;
+		    }
+	            print STDERR "\r" . int($size/1024) . " kb ";
+		} else {
+		    debug("WRITTEN SIZE: " . int($size/1024) . " kb");
 		}
-	        print STDERR "\r" . int($size/1024) . " kb ";
-	    } else {
-		debug("WRITTEN SIZE: " . int($size/1024) . " kb");
-	    }
-	});
+	    });
+	}
 
 	# count the "threads" running here (clerk and scribe)
 	$n_threads = 2;
@@ -784,6 +827,8 @@ sub quit {
 	}
 	if ($self->{'cleanup'}{'roll_trace_log'}) {
 	    log_add_full($L_FINISH, "driver", "fake driver finish");
+	    $self->{'end_time'} = time;
+	    log_add_full($L_FINISH, "amvault", "date $self->{'dst_write_timestamp'} time " . ($self->{'end_time'} - $self->{'start_time'}));
 	    log_add($L_INFO, "pid-done $$");
 
 	    my @amreport_cmd = ("$sbindir/amreport", $self->{'config_name'}, "--from-amdump",
@@ -1066,6 +1111,7 @@ sub usage {
 
 Usage: amvault [-o configoption...] [-q] [--quiet] [-n] [--dry-run]
 	   [--fulls-only] [--latest-fulls] [--incrs-only] [--export]
+	   [--no-interactivity] [--src-labelstr labelstr]
 	   [--src-timestamp src-timestamp] [--exact-match]
 	   --label-template label-template --dst-changer dst-changer
 	   [--autolabel autolabel-arg...]
@@ -1082,6 +1128,8 @@ Usage: amvault [-o configoption...] [-q] [--quiet] [-n] [--dry-run]
     --label-template: the template to use for new volume labels
     --dst-changer: the changer to which dumps should be written
     --autolabel: similar to the amanda.conf parameter; may be repeated (default: empty)
+    --no-interactivity: disable interactivity
+    --str-labelstr: vault from volume with label matching this labelstr
 
 Copies data from the run with timestamp <src-timestamp> onto volumes using
 the changer <dst-changer>, labeling new volumes with <label-template>.  If
@@ -1111,7 +1159,9 @@ my $opt_export = 0;
 my $opt_autolabel = {};
 my $opt_autolabel_seen = 0;
 my $opt_src_write_timestamp;
+my $opt_src_labelstr;
 my $opt_dst_changer;
+my $opt_interactivity = 1;
 
 sub set_label_template {
     usage("only one --label-template allowed") if $opt_autolabel->{'template'};
@@ -1164,7 +1214,9 @@ GetOptions(
     'label-template=s' => \&set_label_template,
     'autolabel=s' => \&add_autolabel,
     'src-timestamp=s' => \$opt_src_write_timestamp,
+    'src-labelstr=s' => \$opt_src_labelstr,
     'dst-changer=s' => \$opt_dst_changer,
+    'interactivity!' => \$opt_interactivity,
     'version' => \&Amanda::Util::version_opt,
     'help' => \&usage,
 ) or usage("usage error");
@@ -1220,6 +1272,7 @@ my $vault = Amvault->new(
     dst_changer => $opt_dst_changer,
     dst_autolabel => $opt_autolabel,
     dst_write_timestamp => Amanda::Util::generate_timestamp(),
+    src_labelstr => $opt_src_labelstr,
     opt_dumpspecs => @opt_dumpspecs? \@opt_dumpspecs : undef,
     opt_dry_run => $opt_dry_run,
     quiet => $opt_quiet,
@@ -1227,6 +1280,7 @@ my $vault = Amvault->new(
     latest_fulls=> $opt_latest_fulls,
     incrs_only => $opt_incrs_only,
     opt_export => $opt_export,
+    opt_interactivity => $opt_interactivity,
     config_overrides_opts => \@config_overrides_opts);
 Amanda::MainLoop::call_later(sub { $vault->run($exit_cb) });
 Amanda::MainLoop::run();
