@@ -1,9 +1,10 @@
 /*
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -89,7 +90,12 @@ cmdline_parse_dumpspecs(
     enum { ARG_GET_HOST, ARG_GET_DISK, ARG_GET_DATESTAMP, ARG_GET_LEVEL } arg_state = ARG_GET_HOST;
 
     while (optind < argc) {
+	char *new_name = NULL;
         name = argv[optind];
+	if (flags & CMDLINE_EXACT_MATCH  && *name != '=') {
+	    new_name = g_strconcat("=", name, NULL);
+	    name = new_name;
+	}
         switch (arg_state) {
             case ARG_GET_HOST:
                 arg_state = ARG_GET_DISK;
@@ -112,12 +118,14 @@ cmdline_parse_dumpspecs(
                 arg_state = ARG_GET_HOST;
 		if (!(flags & CMDLINE_PARSE_LEVEL)) continue;
                 if (name[0] != '\0'
+		    && !(flags & CMDLINE_EXACT_MATCH)
                     && (errstr=validate_regexp(name)) != NULL) {
                     error(_("bad level regex \"%s\": %s\n"), name, errstr);
                 }
                 dumpspec->level = stralloc(name);
                 break;
         }
+	amfree(new_name);
 
 	optind++;
     }
