@@ -130,6 +130,8 @@ static char *errfname = NULL;
 static int   errf_lines = 0;
 static int   max_warnings = 0;
 static char *state_filename = NULL;
+static crc_t crc_data_in;
+static crc_t crc_data_out;
 
 static dumpfile_t file;
 
@@ -385,6 +387,7 @@ main(
     }
 
     safe_cd(); /* do this *after* config_init() */
+    make_crc_table();
 
     check_running_as(RUNNING_AS_ROOT | RUNNING_AS_UID_ONLY);
 
@@ -746,6 +749,7 @@ databuf_flush(
     written = full_write(db->fd, db->dataout,
 			(size_t)(db->datain - db->dataout));
     if (written > 0) {
+	crc32_add((uint8_t *)db->dataout, written, &crc_data_out);
 	db->dataout += written;
         dumpbytes += (off_t)written;
     }
@@ -1774,6 +1778,10 @@ read_datafd(
 	security_stream_close(streams[DATAFD].fd);
 	streams[DATAFD].fd = NULL;
 	aclose(db->fd);
+	g_debug("data in  CRC: %08x:%lld",
+		crc32_finish(&crc_data_in), (long long)crc_data_in.size);
+	g_debug("data out CRC: %08x:%lld",
+		crc32_finish(&crc_data_out), (long long)crc_data_out.size);
 	/*
 	 * If the mesg fd and index fd has also shut down, then we're done.
 	 */
@@ -1782,6 +1790,13 @@ read_datafd(
 	return;
     }
 
+    crc32_add(buf, size, &crc_data_in);
+    if (debug_auth >= 3) {
+	crc_data_in.crc  = crc32_finish(&crc_data_in);
+	g_debug("data in sum CRC: %08x:%lld",
+		crc_data_in.crc, (long long)crc_data_in.size);
+	crc_data_in.crc  = crc32_finish(&crc_data_in);
+    }
     /*
      * We read something.  Add it to the databuf and reschedule for
      * more data.
@@ -2487,6 +2502,8 @@ startup_dump(
     has_maxdumps = am_has_feature(their_features, fe_req_options_maxdumps);
     has_config   = am_has_feature(their_features, fe_req_options_config);
     has_device   = am_has_feature(their_features, fe_sendbackup_req_device);
+    crc32_init(&crc_data_in);
+    crc32_init(&crc_data_out);
 
     /*
      * Default to bsd authentication if none specified.  This is gross.
