@@ -1,7 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-1998 University of Maryland at College Park
- * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
+ * Copyright (c) 2007-2013 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -53,35 +53,29 @@ static GStringChunk *string_chunk = NULL;
 
 find_result_t * find_dump(disklist_t* diskqp) {
     char *conf_logdir, *logfile = NULL;
-    int tape, tape1, maxtape, logs;
+    int tape, maxtape, logs;
     unsigned seq;
-    tape_t *tp, *tp1;
+    tape_t *tp;
     find_result_t *output_find = NULL;
-    gboolean *tape_seen = NULL;
+    GHashTable *tape_seen = g_hash_table_new(g_str_hash, g_str_equal);
 
     if (string_chunk == NULL) {
 	string_chunk = g_string_chunk_new(32768);
     }
     conf_logdir = config_dir_relative(getconf_str(CNF_LOGDIR));
     maxtape = lookup_nb_tape();
-    tape_seen = g_new0(gboolean, maxtape+1);
 
     for(tape = 1; tape <= maxtape; tape++) {
 
-	if (tape_seen[tape] == 1)
-	    continue;
 	tp = lookup_tapepos(tape);
 	if(tp == NULL) continue;
 
-	/* find all tape with the same datestamp */
-	for (tape1 = tape; tape1 <= maxtape; tape1++) {
-	    tp1 = lookup_tapepos(tape1);
-	    if (tp1 == NULL) continue;
-	    if (strcmp(tp->datestamp, tp1->datestamp) != 0)
-		continue;
-
-	    tape_seen[tape1] = 1;
+	/* Do not search the log file if we already searched that datestamp */
+	if (g_hash_table_lookup(tape_seen, tp->datestamp)) {
+	    continue;
 	}
+	g_hash_table_insert(tape_seen, tp->datestamp, GINT_TO_POINTER(1));
+
 
 	/* search log files */
 
@@ -124,7 +118,7 @@ find_result_t * find_dump(disklist_t* diskqp) {
             }
 	}
     }
-    g_free(tape_seen);
+    g_hash_table_destroy(tape_seen);
     amfree(logfile);
     amfree(conf_logdir);
 
@@ -266,7 +260,6 @@ search_holding_disk(
 		continue;
 	    }
 	    dp = add_disk(dynamic_disklist, file.name, file.disk);
-	    enqueue_disk(dynamic_disklist, dp);
 	}
 
 	if(find_match(file.name,file.disk)) {
@@ -1013,7 +1006,6 @@ search_logfile(
 		    continue;
 		}
 		dp = add_disk(dynamic_disklist, host, disk);
-		enqueue_disk(dynamic_disklist, dp);
 	    }
             if (find_match(host, disk)) {
 		if(curprog == P_TAPER) {
