@@ -215,6 +215,7 @@ main(
     planner_setuid = set_root_privs(0);
 
     safe_fd(-1, 0);
+    make_crc_table();
 
     set_pname("planner");
 
@@ -280,7 +281,7 @@ main(
 	diskarg_offset += 1;
     }
     if (argc - diskarg_offset > 0 && g_str_equal(argv[diskarg_offset],
-                                                 "--exact_match")) {
+                                                 "--exact-match")) {
 	exact_match = TRUE;
 	diskarg_offset += 1;
     }
@@ -1068,7 +1069,7 @@ setup_estimate(
 		} else {
 		    ep->degr_mesg = _("Skipping: new disk can't be dumped in degraded mode");
 		}
-	    else
+	    else if (dp->strategy == DS_NOINC)
 		ep->degr_mesg = _("Skipping: strategy NOINC can't be dumped in degraded mode");
 	    if(dp->skip_full) {
 		log_add(L_INFO, _("Ignoring skip-full for %s:%s "
@@ -2555,6 +2556,7 @@ static void delay_dumps(void)
     bi_t  *	nbi;
     gint64	new_total;		/* New total_size */
     char	est_kb[20];		/* Text formatted dump size */
+    char	tape_kb[20];		/* Text formatted tape size */
     int		nb_forced_level_0;
     info_t	info;
     int		delete;
@@ -2605,8 +2607,10 @@ static void delay_dumps(void)
 	}
 
 	/* Format dumpsize for messages */
-	g_snprintf(est_kb, 20, "%lld KB,",
+	g_snprintf(est_kb, 20, "%lld KB",
                    (long long)est(dp)->dump_est->csize);
+	g_snprintf(tape_kb, 20, "%lld KB",
+                   (long long)tapetype_get_length(tape) * (gint64)avail_tapes);
 
 	if(est(dp)->dump_est->level == 0) {
 	    if(dp->skip_incr) {
@@ -2634,8 +2638,9 @@ static void delay_dumps(void)
 	    delete = 1;
 	    message = _("skipping incremental");
 	}
-	delay_one_dump(dp, delete, _("dump larger than available tape space,"),
-		       est_kb, message, NULL);
+	delay_one_dump(dp, delete, "dump estimate (", est_kb,
+		       ") is larger than available tape space (", tape_kb,
+		       "), ", message, NULL);
     }
 
     /*
@@ -2794,7 +2799,7 @@ static void delay_dumps(void)
 	int avail_tapes = 1;
 	nbi = bi->prev;
 	dp = bi->dp;
-	if(dp->splitsize > (gint64)0)
+	if (dp->splitsize > (gint64)0 || dp->allow_split)
 	    avail_tapes = conf_runtapes;
 
 	if(bi->deleted) {
