@@ -30,6 +30,7 @@
 #include <glib-object.h>
 #include "xfer.h"
 #include "amanda.h"
+#include "util.h"
 #include "directtcp.h"
 
 typedef enum {
@@ -153,6 +154,9 @@ typedef struct XferElement {
     /* maximum size to transfer */
     gint64 size;
 
+    /* for crc computation */
+    crc_t crc;
+
     /* if input must be drained in case of write error */
     gboolean must_drain;
     gboolean drain_mode;
@@ -535,4 +539,6 @@ XferElement * xfer_dest_directtcp_connect(DirectTCPAddr *addrs);
  */
 XferElement * xfer_dest_directtcp_listen(void);
 
+int get_err_fd(XferElement *elt);
+
 #endif
